/**
  ******************************************************************************
  * @file    usbd_audio_core.h
  * @author  MCD Application Team
  * @version V1.1.0RC1
  * @date    27-January-2012
  * @brief   header file for the usbd_audio_core.c file.
  ******************************************************************************
  * @attention
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * FOR MORE INFORMATION PLEASE READ CAREFULLY THE LICENSE AGREEMENT FILE
  * LOCATED IN THE ROOT DIRECTORY OF THIS FIRMWARE PACKAGE.
  *
  * <h2><center>&copy; COPYRIGHT 2012 STMicroelectronics</center></h2>
  ******************************************************************************
  */ 

/* Includes ------------------------------------------------------------------*/

#ifndef __USB_AUDIO_CORE_H_
#define __USB_AUDIO_CORE_H_

#include "usbd_ioreq.h"
#include "usbd_req.h"
#include "usbd_desc.h"

#include "stm32_audio_correction.h"

/** @addtogroup STM32_USB_OTG_DEVICE_LIBRARY
  * @{
  */
  
/** @defgroup usbd_audio
  * @brief This file is the Header file for USBD_audio.c
  * @{
  */ 


/** @defgroup usbd_audio_Exported_Defines
  * @{
  */ 

/* Uncomment one of the following define to select the USB Audio Protocol version.
  These two defines are exclusive: only one define can be enabled. */
#define USB_AUDIO_V1_0    /* USB Audio V1.0 */
//#define USB_AUDIO_V2_0    /* USB Audio V2.0 */

/* AudioFreq * DataSize (2 bytes) * NumChannels (Stereo: 2) */
#define DEFAULT_OUT_BIT_RESOLUTION                    16
#define DEFAULT_OUT_CHANNEL_NBR                       2 /* Mono = 1, Stereo = 2 */
#define AUDIO_OUT_PACKET                              (uint32_t)(((DEFAULT_OUT_AUDIO_FREQ * \
                                                                  (DEFAULT_OUT_BIT_RESOLUTION/8) *\
                                                                   DEFAULT_OUT_CHANNEL_NBR) /1000)) 

/* AudioFreq * DataSize (2 bytes) * NumChannels (Stereo: 2) */
#define DEFAULT_IN_BIT_RESOLUTION                    16
#define DEFAULT_IN_CHANNEL_NBR                       1 /* Mono = 1, Stereo = 2 */
#define AUDIO_IN_PACKET                              (uint32_t)(((DEFAULT_IN_AUDIO_FREQ * \
                                                                  (DEFAULT_IN_BIT_RESOLUTION/8) *\
                                                                   DEFAULT_IN_CHANNEL_NBR) /1000)) 

/* Out endpoint max packet size (in Feedback Synchronization mode, this parameter should be set high enough
   in order to tolerate the fluctations of out packets size) */
#ifdef USB_OTG_FS_CORE
 #define AUDIO_OUT_MPS                                 384
#elif defined (USB_OTG_HS_CORE)
 #define AUDIO_OUT_MPS                                 384
#endif /* USB_OTG_FS_CORE */

/* Out endpoint max packet size (in Feedback Synchronization mode, this parameter should be set high enough
   in order to tolerate the fluctations of out packets size) */
#ifdef USB_OTG_FS_CORE
 #define AUDIO_IN_MPS                                 384
#elif defined (USB_OTG_HS_CORE)
 #define AUDIO_IN_MPS                                 384
#endif /* USB_OTG_FS_CORE */

///* Number of sub-packets in the audio transfer buffer. You can modify this value but always make sure
//  that it is an even number and higher than 3 */
//#define OUT_PACKET_NUM                                100
//
//
///* Total size of the audio Out transfer buffer */
//#define OUT_BUF_MARGIN                                4 /* Margin for each sub-buffer */
//#define TOTAL_OUT_BUF_SIZE                            ((uint32_t)((AUDIO_OUT_PACKET + OUT_BUF_MARGIN) * OUT_PACKET_NUM))
//
///* Number of sub-packets in the audio transfer buffer. You can modify this value but always make sure
//  that it is an even number and higher than 3 */
//#define IN_PACKET_NUM                                 20
//
///* Total size of the audio In transfer buffer */
//#define IN_BUF_MARGIN                                 4 /* Margin for each sub-buffer */
//#define TOTAL_IN_BUF_SIZE                             ((uint32_t)((AUDIO_IN_PACKET + IN_BUF_MARGIN) * IN_PACKET_NUM))


#if defined(USB_AUDIO_V1_0)
 #define AUDIO_CONFIG_DESC_SIZE                        (109 + \
                                                        (3 * (SUPPORTED_FREQ_NBR - 1)) + \
                                                        (83 * AUDIO_IN_ENABLED) + \
                                                        (9 ))

#elif defined (USB_AUDIO_V2_0)
 #define AUDIO_CONFIG_DESC_SIZE                        (141 + 7)

#else
 #error "ERROR: define "USB_AUDIO_V1_0" or "USB_AUDIO_V2_0" should be enabled in usbd_audio_core.h"
#endif /* USB_AUDIO_V1_0 */


#define AUDIO_INTERFACE_DESC_SIZE                     9
#define USB_AUDIO_DESC_SIZ                            0x09
#define AUDIO_STANDARD_ENDPOINT_DESC_SIZE             0x09
#define AUDIO_STREAMING_ENDPOINT_DESC_SIZE            0x07

#define AUDIO_DESCRIPTOR_TYPE                         0x21
#define USB_DEVICE_CLASS_AUDIO                        0x01
#define AUDIO_SUBCLASS_AUDIOCONTROL                   0x01
#define AUDIO_SUBCLASS_AUDIOSTREAMING                 0x02
#define AUDIO_PROTOCOL_UNDEFINED                      0x00
#define AUDIO_STREAMING_GENERAL                       0x01
#define AUDIO_STREAMING_FORMAT_TYPE                   0x02

/* Audio Descriptor Types */
#define AUDIO_INTERFACE_DESCRIPTOR_TYPE               0x24
#define AUDIO_ENDPOINT_DESCRIPTOR_TYPE                0x25

/* Audio Control Interface Descriptor Subtypes */
#define AUDIO_CONTROL_HEADER                          0x01
#define AUDIO_CONTROL_INPUT_TERMINAL                  0x02
#define AUDIO_CONTROL_OUTPUT_TERMINAL                 0x03
#define AUDIO_CONTROL_FEATURE_UNIT                    0x06

#define AUDIO_INPUT_TERMINAL_DESC_SIZE                0x0C
#define AUDIO_OUTPUT_TERMINAL_DESC_SIZE               0x09
#define AUDIO_STREAMING_INTERFACE_DESC_SIZE           0x07

#define AUDIO_CONTROL_MUTE                            0x0001

#define AUDIO_FORMAT_TYPE_I                           0x01
#define AUDIO_FORMAT_TYPE_III                         0x03

#define USB_ENDPOINT_TYPE_ISOCHRONOUS                 0x01
#define AUDIO_ENDPOINT_GENERAL                        0x01

#define USB_ENDPOINT_TYPE_ASYNCHRONOUS                0x01
#define USB_ENDPOINT_TYPE_ADAPTIVE                    0x02
#define USB_ENDPOINT_TYPE_SYNCHRONOUS                 0x03

#define AUDIO_REQ_GET_CUR                             0x81
#define AUDIO_REQ_SET_CUR                             0x01

#define AUDIO_REQ_UNDEFINED                           0x00
#define AUDIO_REQ_CUR                                 0x01
#define AUDIO_REQ_RANGE                               0x02
#define AUDIO_REQ_MEM                                 0x03

#define AUDIO_OUT_STREAMING_CTRL                      0x02

#define USB_INTERFACE_ASSOCIATION_TYPE                0x0B
#define AUDIO_CATEGORY_DESKTOP_SPEAKER                0x01
#define AUDIO_CATEGORY_HEADSET                        0x04
#define AUDIO_CATEGORY_MICROPHONE                     0x03
#define AUDIO_CATEGORY_UNDEFINED                      0x00


#define IN_TERMINAL_ID                                1
#define OUT_FU_ID                                     2
#define OUT_TERMINAL_ID                               3
#define OUT_CLOCK_ID                                  4

#define MIC_IN_TERMINAL_ID                            5
#define MIC_FU_ID                                     6
#define MIC_OUT_TERMINAL_ID                           7



/* Buffering state definitions */
#define STATE_IDLE                                    0
#define STATE_BUFFERING                               1
#define STATE_READY                                   2
#define STATE_CLOSING                                 3
#define STATE_RUN                                     4
#define STATE_RUN_OVR                                 (STATE_RUN + 1)
#define STATE_RUN_UDR                                 (STATE_RUN + 2)
#define STATE_RUN_ERR                                 (STATE_RUN + 3)
#define STATE_ERROR                                   8


#define DIRECTION_OUT                                 0
#define DIRECTION_IN                                  1

/* Interrupt feedback enpoint data packet size: 
  In Full Speed: 3 bytes (10.14 format - 10.10 effective data)
  In High Speed: 4 bytes (16.16 format - 10.13 effective data)*/
#define FB_PCKT_SZE                                   3

/* Number of empty frame after which the communication should be considered as closed. */
#define EMPTY_FRAME_THRESHOLD                         3

#define USB_SPEAKER_MAIN_IF_NUM                       0x01
#define USB_SPEAKER_MAIN_IF_ALT_NUM                   0x01


/* Macros to convert the samples number into 10.10 format (extended into 10.14 format) */
#define FEEDBACK_FF_FS_B0(a,b)                        (uint8_t)(((a) & 0x3FC) >> 2)
#define FEEDBACK_FF_FS_B1(a,b)                        (uint8_t)(((((a) & 0x03) << 6) & 0xC0) | (((b) >> 4) & 0x3F))
#ifndef FEEDBACK_EXTENDED_PRECISION_10_14
 #define FEEDBACK_FF_FS_B2(a,b)                        (uint8_t)(((b) << 4) & 0xF0)
#else
 #define FEEDBACK_FF_FS_B2(a,b)                        (uint8_t)(b & 0xFF)
#endif /* FEEDBACK_EXTENDED_PRECISION_10_14 */

/* Security check */
#ifdef USB_AUDIO_V1_0
 #ifdef USB_AUDIO_V2_0
  #error "ERROR: Protocol version define error: only one define can be enabled (USB_AUDIO_V1_0 and USB_AUDIO_V2_0 in usbd_audio_core.h)"
 #endif 
#endif

/**
  * @}
  */ 


///** @defgroup USBD_CORE_Exported_TypesDefinitions
//  * @{
//  */
//typedef struct _Audio_Fops
//{
//    uint8_t   (*Init)         (uint32_t  AudioFreq, uint32_t Volume, uint32_t options);
//    uint8_t   (*DeInit)       (uint32_t options);
//    uint8_t   (*AudioCmd)     (uint8_t* pbuf, uint32_t size, uint8_t cmd);
//    uint8_t   (*VolumeCtl)    (uint8_t vol);
//    uint8_t   (*MuteCtl)      (uint8_t cmd);
//    uint32_t  (*OptionCtrl)   (uint8_t* cmd, uint32_t size);
//    uint8_t   (*GetState)     (void);
//    __IO uint32_t* pOptions;
//}AUDIO_FOPS_TypeDef;


typedef struct _Buff_Desc
{
    uint32_t  Size;
    uint8_t*  Next;
}BUFF_DESC_TypeDef;

/**
  * @}
  */ 



/** @defgroup USBD_CORE_Exported_Macros
  * @{
  */ 
#define AUDIO_PACKET_SZE(frq)          (uint8_t)(((frq * 2 * 2)/1000) & 0xFF), \
                                       (uint8_t)((((frq * 2 * 2)/1000) >> 8) & 0xFF)
#define SAMPLE_FREQ(frq)               (uint8_t)(frq), (uint8_t)((frq >> 8)), (uint8_t)((frq >> 16))

#define BYTE_0(val)                    (uint8_t)(val & 0xFF)
#define BYTE_1(val)                    (uint8_t)((val & 0xFF00) >> 8)
#define BYTE_2(val)                    (uint8_t)((val & 0xFF0000) >> 16)
#define BYTE_3(val)                    (uint8_t)((val & 0xFF000000) >> 24)

/**
  * @}
  */ 

/** @defgroup USBD_CORE_Exported_Variables
  * @{
  */ 

extern USBD_Class_cb_TypeDef  AUDIO_cb;

/**
  * @}
  */ 

/** @defgroup USB_CORE_Exported_Functions
  * @{
  */

/* This function should be called by low layer functions when current buffer transfer is complete */
uint8_t  usbd_audio_BuffXferCplt (uint8_t Direction, uint8_t** pbuf, uint32_t* pSize);

extern void usbd_audio_DescUpdate(uint32_t FBstate);
/**
  * @}
  */ 

#endif  /* __USB_AUDIO_CORE_H_ */
/**
  * @}
  */ 

/**
  * @}
  */ 
  
/******************* (C) COPYRIGHT 2012 STMicroelectronics *****END OF FILE****/
