/**
  ******************************************************************************
  * @file    usbd_audio_synchprocess.c
  * @author  MCD Application Team
  * @version V1.1.0RC1
  * @date    27-January-2012
  * @brief   This file includes the audio correction functional layer
  ******************************************************************************
  * @attention
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * FOR MORE INFORMATION PLEASE READ CAREFULLY THE LICENSE AGREEMENT FILE
  * LOCATED IN THE ROOT DIRECTORY OF THIS FIRMWARE PACKAGE.
  *
  * <h2><center>&copy; COPYRIGHT 2012 STMicroelectronics</center></h2>
  ******************************************************************************
  */ 

/* Includes ------------------------------------------------------------------*/
#include "stm32_audio_synchprocess.h"


#ifdef USE_STM3210C_EVAL
 #include "stm3210c_audio_codec.h"

#elif defined(USE_STM322xG_EVAL)
 #if (AUDIO_SYNCHRO_CLKSYNTH_ENABLED == 1)
  #include "stm322xg_clk_synthesizer_cs2200.h"
 #endif /* AUDIO_SYNCHRO_CLKSYNTH_ENABLED */
  #include "stm322xg_audio_recorder.h"
  #include "stm322xg_audio_codec.h"
 #if (AUDIO_SYNCHRO_SRC_ENABLED != 0)
  #include "stm32f2_resampling.h"
 #endif /* AUDIO_SYNCHRO_SRC_ENABLED */

#elif defined(USE_STM324xG_EVAL)
 #if (AUDIO_SYNCHRO_CLKSYNTH_ENABLED == 1)
  #include "stm324xg_clk_synthesizer_cs2200.h"
 #endif /* AUDIO_SYNCHRO_CLKSYNTH_ENABLED */
  #include "stm324xg_audio_recorder.h"
  #include "stm324xg_audio_codec.h"
 #if (AUDIO_SYNCHRO_SRC_ENABLED != 0)
  #include "stm32f4_resampling.h"
 #endif /* AUDIO_SYNCHRO_SRC_ENABLED */

#elif defined(USE_STM32F4XX_DISCOVERY)
  #include "stm32f4_discovery_audio_recorder.h"
  #include "stm32f4_discovery_audio_codec.h"
 #if (AUDIO_SYNCHRO_SRC_ENABLED != 0)
  #include "stm32f4_resampling.h"
 #endif /* AUDIO_SYNCHRO_SRC_ENABLED */

#endif /* USE_STM3210C_EVAL */



							


/** @addtogroup STM32_Audio_Utilities
* @{
*/

/** @defgroup USBD_AUDIO_SYNCHPROCESS
* @brief    This file includes the audio correction functional layer
* @{
*/ 

/** @defgroup USBD_AUDIO_SYNCHPROCESS_Private_TypesDefinitions
* @{
*/ 
/**
* @}
*/ 


/** @defgroup USBD_AUDIO_SYNCHPROCESS_Private_Defines
* @{
*/ 
/**
* @}
*/ 


/** @defgroup USBD_AUDIO_SYNCHPROCESS_Private_Macros
* @{
*/ 
/**
* @}
*/ 


/** @defgroup USBD_AUDIO_SYNCHPROCESS_Private_Variables
* @{
*/ 

 /*------------------------------- NO SYNCHRO -----------------------------------------*/
SynchStruct_TypeDef SYNCH_NONE_SynchStruct = {
  SYNCH_NONE,                  /* Method Name */
  SYNCH_NONE_PERIOD,           /* Synchro Period */
  SYNCH_NONE_STEP,             /* Synchro step */
  0,                           /* Synchro Threshold */
  NULL,                        /* Synchro option table pointer */
  NULL,                        /* Synchro Init callback */
  NULL,                        /* Synchro Process Callback */
  NULL,                        /* Synchro Update Callback */
  NULL,                        /* Synchro DeInit Callback */
};

#if (AUDIO_SYNCHRO_CLKSYNTH_ENABLED == 1)
 /*-------------------------- Extern Clock SYNCHRO -----------------------------------*/
uint8_t tOptTable[5] = {
  DEFAULT_INJECTED_ERROR,        /* Enable or disable the injected error on the output clock */
  SYNCH_EXTCLK_PERIOD_ALT,       /* Synhcronization period when injected error is enabled */
  SYNCH_EXTCLK_STEP_ALT,         /* Synhcronization step when injected error is enabled */
  USE_DYNAMIC_ADJUSTMENT_FACTOR, /* 0: use the dynamic adjustment factor, !0: don't use */ 
  DYNAMIC_ADJUSTMENT_FACTOR_MAX, /* Maximum adjustement factor value */
};

static uint32_t CLK_SYNTH_Process (uint32_t* pValue, uint32_t* pDirection, uint32_t* pBuf);

/* Main exported structure */
SynchStruct_TypeDef SYNCH_EXTCLK_SynchStruct = {
  SYNCH_EXTCLK,                /* Method Name */
  SYNCH_EXTCLK_PERIOD,         /* Synchro Period */
  SYNCH_EXTCLK_STEP,           /* Synchro step */
  SYNCH_EXTCLK_THRESHOLD,      /* Synchro Threshold */
  tOptTable,                   /* Synchro option table pointer */
  CLK_SYNTH_Init,              /* Synchro Init callback */
  CLK_SYNTH_Process,           /* Synchro Process Callback */
  CLK_SYNTH_UpdateFreq,        /* Synchro Update Callback */
  CLK_SYNTH_DeInit,            /* Synchro DeInit Callback */
};
#endif /* AUDIO_SYNCHRO_CLKSYNTH_ENABLED */


 /*-------------------------- FEEDBACK SYNCHRO -------------------------------------*/
#if (AUDIO_SYNCHRO_FEEDBACK_ENABLED == 1)
uint8_t tFeedbackOptTable[1] = {
  FEEDBACK_REFRESH,              /* Enable or disable the injected error on the output clock */
};

static uint32_t SYNCH_FEEDBACK_Process (uint32_t* pValue, uint32_t* pDirection, uint32_t* pBuf);

/* Main exported structure */
SynchStruct_TypeDef SYNCH_FEEDBACK_SynchStruct = {
  SYNCH_FEEDBACK,              /* Method Name */
  SYNCH_FEEDBACK_PERIOD,       /* Synchro Period */
  SYNCH_FEEDBACK_STEP,         /* Synchro step */
  SYNCH_FEEDBACK_THRESHOLD,    /* Synchro Threshold */
  tFeedbackOptTable,           /* Synchro option table pointer */
  NULL,                        /* Synchro Init callback */
  SYNCH_FEEDBACK_Process,      /* Synchro Process Callback */
  NULL,                        /* Synchro Update Callback */
  NULL,                        /* Synchro DeInit Callback */
};
#endif /* AUDIO_SYNCHRO_FEEDBACK_ENABLED */

 /*-------------------------- ADD-REM SYNCHRO -----------------------------------------*/
#if (AUDIO_SYNCHRO_ADDREM_ENABLED == 1)
static uint32_t SYNCH_ADDREM_Process (uint32_t* pValue, uint32_t* pDirection, uint32_t* pBuf);

/* Main exported structure */
SynchStruct_TypeDef SYNCH_ADDREM_SynchStruct = {
  SYNCH_ADDREM,                /* Method Name */
  SYNCH_ADDREM_PERIOD,         /* Synchro Period */
  SYNCH_ADDREM_STEP,           /* Synchro step */
  NULL,                        /* Synchro Threshold */
  NULL,                        /* Synchro option table pointer */
  NULL,                        /* Synchro Init callback */
  SYNCH_ADDREM_Process,        /* Synchro Process Callback */
  NULL,                        /* Synchro Update Callback */
  NULL,                        /* Synchro DeInit Callback */
};
#endif /* AUDIO_SYNCHRO_ADDREM_ENABLED */

 /*----------------------------- SRC SYNCHRO -----------------------------------------*/
#ifdef AUDIO_SYNCHRO_SRC_ENABLED
uint8_t tSrcOptTable[1] = {
  0,            /* For future use */ 
};

static uint32_t SYNCH_SRC_Process (uint32_t* pValue, uint32_t* pDirection, uint32_t* pBuf);
static uint32_t SYNCH_SRC_Init(uint32_t p1, uint32_t p2, uint32_t p3, uint32_t p4);

/* Main exported structure */
SynchStruct_TypeDef SYNCH_SRC_SynchStruct = {
  SYNCH_SRC,              /* Method Name */
  SYNCH_SRC_PERIOD,       /* Synchro Period */
  SYNCH_SRC_STEP,         /* Synchro step */
  NULL,                   /* Synchro Threshold */
  tSrcOptTable,           /* Synchro option table pointer */
  //STM32_Resample_Init,    /* Synchro Init callback */
  SYNCH_SRC_Init,         /* Synchro Init callback */
  SYNCH_SRC_Process,      /* Synchro Process Callback */
  NULL,                   /* Synchro Update Callback */
  NULL,                   /* Synchro DeInit Callback */
};

/////////////////////////////SRC Method 5
#define SRC_RATIOS_NUMBER          3//201  //41
#define SRC_RATIO_IDX_1            1//102  //22
#define SRC_STEP_MAX               1//10
#define SRC_STEP_MIN               1

typedef enum
{
  SRC_IDLE = 0,
  SRC_UP,
  SRC_DOWN
}SRC_STATE_TypeDef;

typedef enum
{
  SRC_FREE = 0,
  SRC_RUN,
  SRC_DONE
}SRC_MODE_TypeDef;

__IO SRC_STATE_TypeDef SRCState = SRC_IDLE, SRCStatePrev = SRC_IDLE;
__IO uint32_t SRCStep = 1; //10;
__IO uint32_t SRCRatioIdx = SRC_RATIO_IDX_1;
__IO SRC_MODE_TypeDef SRCMode = SRC_FREE;
__IO uint32_t SRCCounter = 0;

__IO uint32_t PrevDir = 0;
__IO uint32_t Blocked = 0;
__IO uint32_t Factor = 1;
  
  
__IO uint32_t RatioStep = (SRC_RATIO_MAX - RATIO_1)/4;
__IO uint32_t RatioBlockTime = SRC_BLOCK_THRESHOLD;
__IO int32_t  DirectionIdx = 0;
#endif /* AUDIO_SYNCHRO_SRC_ENABLED */

#if (USB_AUDIO_IN_SYNCHRO_CLKUPDT_ENABLED == 1)
 /*-------------------------- CLKUPDT SYNCHRO -----------------------------------------*/
static uint32_t SYNCH_CLKUPDT_Process (uint32_t* pValue, uint32_t* pDirection, uint32_t* pBuf);

/* Main exported structure */
SynchStruct_TypeDef SYNCH_CLKUPDT_SynchStruct = {
  SYNCH_CLKUPDT,               /* Method Name */
  SYNCH_CLKUPDT_PERIOD,        /* Synchro Period */
  SYNCH_CLKUPDT_STEP,          /* Synchro step */
  NULL,                        /* Synchro Threshold */
  NULL,                        /* Synchro option table pointer */
  NULL,                        /* Synchro Init callback */
  SYNCH_CLKUPDT_Process,       /* Synchro Process Callback */
  NULL,                        /* Synchro Update Callback */
  NULL,                        /* Synchro DeInit Callback */
};
 /*-----------------------------------------------------------------------------------*/
#endif /* USB_AUDIO_IN_SYNCHRO_CLKUPDT_ENABLED */

/* Table for Out stream synchronization methods list */
SynchStruct_TypeDef*  OutMethTab[] = {
  &SYNCH_NONE_SynchStruct,
 #if (AUDIO_SYNCHRO_CLKSYNTH_ENABLED == 1)
  &SYNCH_EXTCLK_SynchStruct,
 #endif
 #if (AUDIO_SYNCHRO_FEEDBACK_ENABLED == 1)
  &SYNCH_FEEDBACK_SynchStruct,
 #endif
 #if (AUDIO_SYNCHRO_ADDREM_ENABLED == 1)
  &SYNCH_ADDREM_SynchStruct,
 #endif
 #ifdef AUDIO_SYNCHRO_SRC_ENABLED
  &SYNCH_SRC_SynchStruct,
 #endif /* AUDIO_SYNCHRO_SRC_ENABLED */
  
};

/* Table for In stream synchronization methods list */
SynchStruct_TypeDef*  InMethTab[] = {
  &SYNCH_NONE_SynchStruct,
#if (USB_AUDIO_IN_SYNCHRO_CLKUPDT_ENABLED == 1)
  &SYNCH_CLKUPDT_SynchStruct,
#endif
  
};

/**
* @}
*/

/** @defgroup USBD_AUDIO_SYNCHPROCESS_Private_Constants
* @{
*/ 

/**
* @}
*/

/** @defgroup USBD_AUDIO_SYNCHPROCESS_Private_FunctionPrototypes
* @{
*/

/**
* @}
*/ 


/** @defgroup USBD_AUDIO_SYNCHPROCESS_Private_Functions
* @{
*/ 

#if (AUDIO_SYNCHRO_CLKSYNTH_ENABLED == 1)
/**
  * @brief  Processes the audio correction.
  * @param  Value: value of the shift to be corrected.
  * @param  pDirection: direction of the shift (up or down).
  * @param  pBuf: optional parameter (not used for this method).
  * @retval 0 if passed. !0 if error occured during operation.
  */
static uint32_t CLK_SYNTH_Process (uint32_t* pValue, uint32_t* pDirection, uint32_t* pBuf)
{  
  uint32_t result = 0;
  uint32_t factor = 1;
  
  if (SYNCH_EXTCLK_SynchStruct.pSynchOpt[3])
  {
    /* Get the adjustment factor. A maximum factor value of 4 is accepted (4 x 5ppm = 20ppm) */
    factor = ((*pValue) > SYNCH_EXTCLK_SynchStruct.pSynchOpt[4])? SYNCH_EXTCLK_SynchStruct.pSynchOpt[4]:(*pValue);
    
    /* Make sure that the  adjustment factor is not 0 */
    if (factor == 0)
    {
      factor = 1;
    }
  }
  
  /* Adjustment mechanism to correct the overrun/underrun situation */
  if (*pDirection)
  { /* Overrun condition */    
    /* Adjust the clock frequency to speed up output frequency */
    result = CLK_SYNTH_AdjustPpm((SYNCH_EXTCLK_SynchStruct.SynchStep * factor), EXTCLK_DIR_UP);
  }
  else
  {/* Underrun condition */    
    /* Adjust the clock frequency to slow down output frequency */
    result = CLK_SYNTH_AdjustPpm((SYNCH_EXTCLK_SynchStruct.SynchStep * factor), EXTCLK_DIR_DOWN);
  }
  
  /* Return the result of operations */
  return result;
}
#endif /* AUDIO_SYNCHRO_CLKSYNTH_ENABLED */

#if (AUDIO_SYNCHRO_FEEDBACK_ENABLED == 1)
/**
  * @brief  Processes the audio correction.
  * @param  Value: value of the shift to be corrected.
  * @param  pDirection: direction of the shift (up or down).
  * @param  pBuf: optional parameter (for future use).
  * @retval 0 if passed. !0 if error occured during operation.
  */
static uint32_t SYNCH_FEEDBACK_Process (uint32_t* pValue, uint32_t* pDirection, uint32_t* pBuf)
{
  static uint32_t count = 0;
  
  /* Adjustment mechanism to correct the overrun/underrun situation */
  if (*pDirection)
  { /* Overrun condition */    
    if (++count < SYNCH_FEEDBACK_SynchStruct.SynchThreshold)
    {
      /* Adjust the feedback value */
      if ((*(uint32_t *)(pBuf + 1)) < SYNCH_FEEDBACK_SynchStruct.SynchStep)
      {
        (*pBuf) --;
        (*(uint32_t *)(pBuf + 1)) = MAX_FRACTION;
      }   
      else
      {
        (*(uint32_t *)(pBuf + 1)) -= SYNCH_FEEDBACK_SynchStruct.SynchStep;
      }
    }
    else
    {
      (*pBuf) = (*(uint32_t *)(pBuf + 2));
      (*(uint32_t *)(pBuf + 1)) = (*(uint32_t *)(pBuf + 3));
      count = 0;
    } 
  }
  else
  {/* Underrun condition */
    if (++count < SYNCH_FEEDBACK_SynchStruct.SynchThreshold)
    {    
      /* Adjust the feedback value */
      if ((*(uint32_t *)(pBuf + 1)) > (MAX_FRACTION - SYNCH_FEEDBACK_SynchStruct.SynchStep))
      {
        (*pBuf) ++;
        (*(uint32_t *)(pBuf + 1)) = 0;
      }
      else
      {
        (*(uint32_t *)(pBuf + 1)) += SYNCH_FEEDBACK_SynchStruct.SynchStep;
      }
    }
    else
    {
      (*pBuf) = (*(uint32_t *)(pBuf + 2));
      (*(uint32_t *)(pBuf + 1)) = (*(uint32_t *)(pBuf + 3));
      count = 0;
    }   
  }
  
  return 0;
}
#endif /* AUDIO_SYNCHRO_FEEDBACK_ENABLED */

#if (AUDIO_SYNCHRO_ADDREM_ENABLED == 1)
/**
  * @brief  Processes the audio correction.
  * @param  Value: value of the shift to be corrected.
  * @param  pDirection: direction of the shift (up or down).
  * @param  pBuf: optional parameter (for future use).
  * @retval 0 if passed. !0 if error occured during operation.
  */
uint32_t SYNCH_ADDREM_Process(uint32_t* pValue, uint32_t* pDirection, uint32_t* pBuf)
{  
  /* Adjustment mechanism to correct the overrun/underrun situation */
  if (*pDirection)
  { /* Overrun condition */
    (*pBuf) -= SYNCH_ADDREM_SynchStruct.SynchStep;
  }
  else
  {/* Underrun condition */
    (*pBuf) += SYNCH_ADDREM_SynchStruct.SynchStep;   
  }
  
  return 0;
}
#endif /* AUDIO_SYNCHRO_ADDREM_ENABLED */

#if (AUDIO_SYNCHRO_SRC_ENABLED == 1)

#define SRC_SHIFT_PHASE_NBR                        5
#define SRC_SHIFT_PHASE_MAX                        3000
#define SRC_SHIFT_PHASE_MIN                        10

uint32_t ShiftPhaseTab[SRC_SHIFT_PHASE_NBR] = {
                 0, 0, 0, 0, 0//, 0, 0, 0, 0, 0
                 };
uint32_t ShiftPhaseMean = 0;
uint32_t ShiftPhasePrev = 0;
uint32_t ShiftPhaseCount = 0;

/**
  * @brief  Initializes the SRC module.
  * @param  p1 to p4 not used
  * @retval 0 if passed. !0 if error occured during operation.
  */
uint32_t SYNCH_SRC_Init(uint32_t p1, uint32_t p2, uint32_t p3, uint32_t p4)
{
  uint32_t result = 0;
  
  NVIC_InitTypeDef NVIC_InitStruct;

  result = STM32_Resample_Init(0, 0, 0, 0);
  
  NVIC_InitStruct.NVIC_IRQChannel = SRC_IRQ;
  NVIC_InitStruct.NVIC_IRQChannelPreemptionPriority = 1;
  NVIC_InitStruct.NVIC_IRQChannelSubPriority = 1;
  NVIC_InitStruct.NVIC_IRQChannelCmd = ENABLE;
    
  NVIC_Init(&NVIC_InitStruct);
  
  return result;
}


/**
  * @brief  Processes the audio correction.
  * @param  Value: value of the shift to be corrected.
  * @param  pDirection: direction of the shift (up or down).
  * @param  pBuf: optional parameter (for future use).
  * @retval 0 if passed. !0 if error occured during operation.
  */
static uint32_t SYNCH_SRC_Process (uint32_t* pValue, uint32_t* pDirection, uint32_t* pBuf)
{     
  static uint32_t ShiftPrev = 0;

  /* Adjustment mechanism to correct the overrun/underrun situation */
  
  ////////Near to the optimum value (experimental), for reference only !!
  ////(*pBuf) = 268463421; //RATIO_48_TO_47_995;
  ////return 0;

  /* If no buffer is passed to the function, reset all parameters to default values */  
  if (pBuf == 0)
  {
    uint32_t idx = 0;
    
    ShiftPrev = 0;
     
    for (idx = 0; idx < SRC_SHIFT_PHASE_NBR; idx++)
    {
      ShiftPhaseTab[idx] = 0;
    }
    
    ShiftPhaseMean = 0;
    ShiftPhasePrev = 0;
    ShiftPhaseCount = 0;
    
    Factor = 1;
    SYNCH_SRC_SynchStruct.SynchPeriod = SYNCH_SRC_PERIOD;
    RatioStep = 0;
    
    return 0;
  }  
  
  /* Get the difference between previous and current shift value */
  ShiftPhaseTab[ShiftPhaseCount++] = ShiftPrev;

  /* Update the previous value with the current one */
  ShiftPrev = (*pValue);
  
  /* Get mean value of shift phase */
  if (ShiftPhaseCount >= SRC_SHIFT_PHASE_NBR)
  {
    ShiftPhaseMean = 0;
    do
    {
      ShiftPhaseMean += ShiftPhaseTab[--ShiftPhaseCount];
    }while (ShiftPhaseCount);
    ShiftPhaseMean /= SRC_SHIFT_PHASE_NBR;
  }
  else
  {
    return 0;
  }

  /* In case of peak shifts, reset state machine and use maximum values */
  ///Is it necesseray ???!! TBC
  if (ShiftPhaseMean > SRC_SHIFT_PHASE_MAX)
  {
    /* Reset the factor to default value */
    Factor = 1;
    /* Reset the period to default value */
    SYNCH_SRC_SynchStruct.SynchPeriod = SYNCH_SRC_PERIOD;
  }
  

  /* Number of Samples shifted for 1 Second */
  RatioStep = (((ShiftPhaseMean * 100) / SYNCH_SRC_SynchStruct.SynchPeriod) * SRC_RATIO_STEP_0_001) / 10;
      
  /* Overrun condition ---------------------------*/
  if (*pDirection)
  {
    /* Set new ratio value */
    (*pBuf) += (RatioStep / Factor);
  } 
  /* Underrun condition --------------------------*/
  else
  {
    /* Set new ratio value */
    (*pBuf) -= (RatioStep / Factor);
  }
  
  /* Guarentee that ratio doesn't exceed max and min values to avoid
     fast divergence */
  if ((*pBuf) > SRC_RATIO_MAX)
  {
    (*pBuf) = SRC_RATIO_MAX;
  }
  if ((*pBuf) < SRC_RATIO_MIN)
  {
    (*pBuf) = SRC_RATIO_MIN;
  }
  
  /* Both streams are synchronized, no correction to be performed, slow down the synchro period */
  if (ShiftPhaseMean == 0)
  {
    if (SYNCH_SRC_SynchStruct.SynchPeriod <= SRC_SYNC_PERIOD_MAX)
    {
      SYNCH_SRC_SynchStruct.SynchPeriod += SRC_SYNC_PERIOD_UP;
    }
      
    if (Factor < SRC_SYNC_FACTOR_MAX)
    {
      Factor *= SRC_SYNC_FACTOR_HIGH;
    }
  }
  
  /* Streams are not synchronized, correction should be performed */
  else
  {
    /* If no shift, keep all values */
    if (ShiftPhasePrev == 0)
    {
      ShiftPhasePrev = ShiftPhaseMean;
    }
    
    /* If system is converging to the optimum ratio, reduce change dynamic */
    else if (ShiftPhasePrev > ShiftPhaseMean)
    {
      if (SYNCH_SRC_SynchStruct.SynchPeriod >= SRC_SYNC_PERIOD_MIN)
      {
        SYNCH_SRC_SynchStruct.SynchPeriod -= SRC_SYNC_PERIOD_DOWN;
      }
      
      if (Factor > SRC_SYNC_FACTOR_MIN)
      {
        Factor /= SRC_SYNC_FACTOR_LOW;
        Factor = (Factor)? Factor:1 ;
      }
    }
    
    /* If system is diverging from the optimum ratio, speed up change dynamic */
    else if (ShiftPrev <= ShiftPhaseMean)
    {
      if (SYNCH_SRC_SynchStruct.SynchPeriod <= SRC_SYNC_PERIOD_MAX)
      {
        SYNCH_SRC_SynchStruct.SynchPeriod += SRC_SYNC_PERIOD_UP;
      }
        
      if (Factor < SRC_SYNC_FACTOR_MAX)
      {
        Factor *= SRC_SYNC_FACTOR_LOW;
      }
    } 
  }
  
  /* Store previous shift value */
  ShiftPhasePrev = ShiftPhaseMean;
  
  return 0;
}

///////////////////Stable function, kept commented for backup prupose only
///**
//  * @brief  Processes the audio correction.
//  * @param  Value: value of the shift to be corrected.
//  * @param  pDirection: direction of the shift (up or down).
//  * @param  pBuf: optional parameter (for future use).
//  * @retval 0 if passed. !0 if error occured during operation.
//  */
//static uint32_t SYNCH_SRC_Process (uint32_t* pValue, uint32_t* pDirection, uint32_t* pBuf)
//{     
//  static uint32_t ShiftPrev = 0;
////  static uint32_t ShiftPhase = 0;
//
//  /* Adjustment mechanism to correct the overrun/underrun situation */
//  
//  ////////Near to the optimum value (experimental), for reference only !!
//  ////(*pBuf) = 268463421; //RATIO_48_TO_47_995;
//  ////return 0;
//
//  /* Get the difference between previous and current shift value */
//  //ShiftPhaseTab[ShiftPhaseCount++] = (ShiftPrev > (*pValue))? (ShiftPrev - (*pValue)):((*pValue) - ShiftPrev);
//  ShiftPhaseTab[ShiftPhaseCount++] = ShiftPrev;
//
//  /* Update the previous value with the current one */
//  ShiftPrev = (*pValue); 
//  
////  /* Discard noise */
////  if (ShiftPhaseTab[ShiftPhaseCount - 1] > SRC_SHIFT_PHASE_MAX)
////  {
////    ShiftPhaseCount--;
////    return 0;
////  }
//
////  /* Discard noise */
////  if ((ShiftPhaseTab[ShiftPhaseCount - 1] > (ShiftPhaseMean * 100)) && (ShiftPhaseMean > 0))
////  {
////    ShiftPhaseCount--;
////    return 0;
////  }
//  
//  /* Get mean value of shift phase */
//  if (ShiftPhaseCount >= SRC_SHIFT_PHASE_NBR)
//  {
//    ShiftPhaseMean = 0;
//    do
//    {
//      ShiftPhaseMean += ShiftPhaseTab[--ShiftPhaseCount];
//    }while (ShiftPhaseCount);
//    ShiftPhaseMean /= SRC_SHIFT_PHASE_NBR;
//  }
//  else
//  {
//    return 0;
//  }
//
//  /* In case of peak shifts, reset state machine and use maximum values */
//  ///Is it necesseray ???!! TBC
//  if (ShiftPhaseMean > SRC_SHIFT_PHASE_MAX)
//  {
//    /* Reset the factor to default value */
//    Factor = 1;
//    /* Reset the period to default value */
//    SYNCH_SRC_SynchStruct.SynchPeriod = SYNCH_SRC_PERIOD;
//  }
// 
////  if (ShiftPhaseMean < SRC_SHIFT_PHASE_MIN)
////  {
////    if (SYNCH_SRC_SynchStruct.SynchPeriod <= 1000)
////    {
////      SYNCH_SRC_SynchStruct.SynchPeriod += 10;
////    }
////      
////    if (Factor < 256)
////    {
////      Factor *= 2;
////    }
////  }
//  
//
//  /* Number of Samples shifted for 1 Second */
//  //RatioStep = ((((*pValue) * 100) / SYNCH_SRC_SynchStruct.SynchPeriod) * SRC_RATIO_STEP_0_001) / 10;
//  RatioStep = (((ShiftPhaseMean * 100) / SYNCH_SRC_SynchStruct.SynchPeriod) * SRC_RATIO_STEP_0_001) / 10;
// 
// 
// 
////  if (ShiftPrev == 0)
////  {
////    ShiftPrev = (*pValue);
////  }
////  else if ((*pValue) > ShiftPrev)
////  {
////    RatioStep *= 2;
////    RatioBlockTime /= 2;
////  }
////  else if ((*pValue) < ShiftPrev)
////  {
////    if (RatioStep > 1)
////    {
////      RatioStep /= 2;
////    }
////    if (RatioBlockTime < SRC_MAX_BLOCK_TIME)
////    {
////      RatioBlockTime *= 2;
////    } 
////  }
//
////  /* Decrement and check blocking threshold var */
////  if (Blocked > 0)
////  {
////    Blocked--;
////    return 0;
////  }
////  
////  /* Reset the blocking threshold var to default */
////  Blocked = RatioBlockTime;
//      
//  /* Overrun condition ---------------------------*/
//  if (*pDirection)
//  {
//    /* Set new ratio value */
////    (*pBuf) = RATIO_1 + RatioStep;
//    (*pBuf) += (RatioStep / Factor);
////    (*pBuf) += RatioStep;
////    DirectionIdx ++;
//  } 
//  /* Underrun condition --------------------------*/
//  else
//  {
//    /* Set new ratio value */
////    (*pBuf) = RATIO_1 - RatioStep;
//    (*pBuf) -= (RatioStep / Factor);
////    (*pBuf) -= RatioStep;
////    DirectionIdx --;
//  }
//  
//  /* Guarentee that ratio doesn't exceed max and min values to avoid
//     fast divergence */
//  if ((*pBuf) > SRC_RATIO_MAX)
//  {
//    (*pBuf) = SRC_RATIO_MAX;
//  }
//  if ((*pBuf) < SRC_RATIO_MIN)
//  {
//    (*pBuf) = SRC_RATIO_MIN;
//  }
//
// 
//  if (ShiftPhaseMean == 0)
//  {
//    if (SYNCH_SRC_SynchStruct.SynchPeriod <= 1000)
//    {
//      SYNCH_SRC_SynchStruct.SynchPeriod += 50;
//    }
//      
//    if (Factor < 256)
//    {
//      Factor *= 4;
//    }
//  }
//  else
//  {
// 
//    /* If no shift, keep all values */
//    if (ShiftPhasePrev == 0)
//    {
//      ShiftPhasePrev = ShiftPhaseMean;
//    }
//    
//    /* If system is converging to the optimum ratio, slow down change dynamic */
//    else if (ShiftPhasePrev > ShiftPhaseMean)
//    //else if (ShiftPrev > ShiftPhaseMean)
//    {
//      if (SYNCH_SRC_SynchStruct.SynchPeriod >= 200)
//      {
//        SYNCH_SRC_SynchStruct.SynchPeriod -= 10;
//      }
//      
//      if (Factor > 1)
//      {
//        Factor /= 2;
//      }
//    }
//    
//    /* If system is diverging from the optimum ratio, speed up change dynamic */
//    //else if (ShiftPhasePrev <= ShiftPhaseMean)
//    else if (ShiftPrev <= ShiftPhaseMean)
//    {
//      if (SYNCH_SRC_SynchStruct.SynchPeriod <= 1000)
//      {
//        SYNCH_SRC_SynchStruct.SynchPeriod += 10;
//      }
//        
//      if (Factor < 256)
//      {
//        Factor *= 2;
//      }
//    } 
//  }
//  
//  /* Store previous shift value */
//  ShiftPhasePrev = ShiftPhaseMean;
//  //ShiftPrev = ShiftPhaseMean;
//
//
//
////  if (ShiftPrev == 0)
////  {
////    ShiftPrev = (*pValue);
////  }
////  else if ((*pValue) > ShiftPrev)
////  {
////    if (((*pValue) - ShiftPrev) >= ((*pValue)/10))
////    {
////      if (SYNCH_SRC_SynchStruct.SynchPeriod >= 200)
////      {
////        SYNCH_SRC_SynchStruct.SynchPeriod -= 10;
////      }
////      
////      if (Factor > 1)
////      {
////        Factor /= 2;
////      }
////    }
////  }
////  else if ((*pValue) < ShiftPrev)
////  {
////    if ((ShiftPrev - (*pValue)) >= (ShiftPrev/10))
////    {  
////      if (SYNCH_SRC_SynchStruct.SynchPeriod <= 1000)
////      {
////        SYNCH_SRC_SynchStruct.SynchPeriod += 10;
////      }
////        
////      if (Factor < 128)
////      {
////        Factor *= 2;
////      }
////    } 
////  }
////  
////  /* Update the previous value with the current one */
////  ShiftPrev = (*pValue);  
//    
//    
////  if ((DirectionIdx > 1) || (DirectionIdx < -1))
////  {
////    RatioStep /= 2;
////    RatioBlockTime *= 2;
////  }
//  
//
//
//
//
//
//  
////  /* Overrun condition ---------------------------*/
////  if (*pDirection)
////  {   
////    /* Decrement and check blocking threshold var */
////    if (Blocked > 0)
////    {
////      Blocked--;
////      return 0;
////    }
////    
////    /* Reset the blocking threshold var to default */
////    Blocked = SRC_BLOCK_THRESHOLD;
////      
////    /* If current direction is opposite to previous, reshape factor */
//////    if (PrevDir != 1)
//////    { 
////      switch (Factor)
////      {
////      case 1:
////        Factor = 10;
////        break;
////      case 10:
////        Factor = 100;
////        break;
////      case 100:
////        Factor = 1000;
////        break;
////      case 1000:
////        break;
////      default:
////        Factor = 1;
////        break;     
////      }
//////    }
////    
////    /* Set new ratio value */
////    (*pBuf) += (SRC_RATIO_STEP_0_001 / Factor);
////    
////    /* Update the previous direction with current */
////    PrevDir = 1;
////    
////    SRCCounter++;
////  }
////  
////  /* Underrun condition --------------------------*/
////  else
////  {
////    /* Decrement and check blocking threshold var */
////    if (Blocked > 0)
////    {
////      Blocked--;
////      return 0;
////    }
////    
////    /* Reset the blocking threshold var to default */
////    Blocked = SRC_BLOCK_THRESHOLD;
////    
////    /* If current direction is opposite to previous, reshape factor */
//////    if (PrevDir)
//////    { 
////      switch (Factor)
////      {
////      case 1:
////        Factor = 10;
////        break;
////      case 10:
////        Factor = 100;
////        break;
////      case 100:
////        Factor = 1000;
////        break;
////      case 1000:
////        break;
////      default:
////        Factor = 1;
////        break;     
////      }
//////    }
////    
////    /* Set new ratio value */
////    (*pBuf) -= 10; //(SRC_RATIO_STEP_0_001 / Factor);
////    
////    /* Update the previous direction with current */
////    PrevDir = 0;
////    
////    SRCCounter++;
////  }
////
////
////  if (SRCCounter++ >= 3000)
////  {
////    
////  }
//  
//
//
//  
////  if (*pDirection)
////  { /* Overrun condition */
////
////    /* First level of correction */
////    if ((SRCCounter) < SRC_RATIO_THRESHOLD/2)
////    {
////      (*pBuf) += SRC_RATIO_STEP_0_001*10;
////    }
////    /* Second level of correction */
////    else if ((SRCCounter) < (SRC_RATIO_THRESHOLD*3))
////    {
////      (*pBuf) += SRC_RATIO_STEP_0_001;
////    }
////    /* Third level of correction */
////    else if ((SRCCounter) < (SRC_RATIO_THRESHOLD*5))
////    {
////      (*pBuf) += SRC_RATIO_STEP_0_001/10;
////    }
////  }
////  else
////  {/* Underrun condition */
////  
////    /* First level of correction */
////    if ((SRCCounter) < SRC_RATIO_THRESHOLD/2)
////    {
////      (*pBuf) -= SRC_RATIO_STEP_0_001*10;
////    }
////    /* Second level of correction */
////    else if ((SRCCounter) < (SRC_RATIO_THRESHOLD*3))
////    {
////      (*pBuf) -= SRC_RATIO_STEP_0_001;
////    }
////    /* Third level of correction */
////    else if ((SRCCounter) < (SRC_RATIO_THRESHOLD*5))
////    {
////      (*pBuf) -= SRC_RATIO_STEP_0_001/10;
////    } 
////  }
////
////  /* Increment counter */
////  SRCCounter++;
//  
//  /* Limit the ratio to the maximum and minimum values*/
////  if ((*pBuf) > SRC_RATIO_MAX)
////  {
////    (*pBuf) = SRC_RATIO_MAX;
////    SRCCounter = 0;
////  }
////  if ((*pBuf) > SRC_RATIO_MIN)
////  {
////    (*pBuf) = SRC_RATIO_MIN;
////    SRCCounter = 0;
////  }
//  
//  return 0;
//}
#endif /* AUDIO_SYNCHRO_SRC_ENABLED */

#if (USB_AUDIO_IN_SYNCHRO_CLKUPDT_ENABLED == 1)
/**
  * @brief  Processes the audio correction.
  * @param  Value: value of the shift to be corrected.
  * @param  pDirection: direction of the shift (up or down).
  * @param  pBuf: optional parameter (for future use).
  * @retval 0 if passed. !0 if error occured during operation.
  */
static uint32_t SYNCH_CLKUPDT_Process (uint32_t* pValue, uint32_t* pDirection, uint32_t* pBuf)
{    
  /* Adjustment mechanism to correct the overrun/underrun situation */
  if (*pDirection)
  { /* Overrun condition */  
    /* Adjust the clock frequency to speed up output frequency */
    STM32_AudioRec_ClkUpdate(SYNCH_CLKUPDT_SynchStruct.SynchStep, AUDIO_REC_DIRECTION_DOWN);
  }
  else
  {/* Underrun condition */    
    /* Adjust the clock frequency to speed up output frequency */
    STM32_AudioRec_ClkUpdate(SYNCH_CLKUPDT_SynchStruct.SynchStep, AUDIO_REC_DIRECTION_UP);
  }
  
  return 0;
}
#endif /* USB_AUDIO_IN_SYNCHRO_CLKUPDT_ENABLED */

/**
* @}
*/ 

/**
* @}
*/ 

/******************* (C) COPYRIGHT 2012 STMicroelectronics *****END OF FILE****/
