/**
  ******************************************************************************
  * @file    audio_app_conf.h
  * @author  MCD Application Team
  * @version V1.0.0RC2
  * @date    13-February-2012
  * @brief   USB Device configuration file
  ******************************************************************************
  * @attention
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * FOR MORE INFORMATION PLEASE READ CAREFULLY THE LICENSE AGREEMENT FILE
  * LOCATED IN THE ROOT DIRECTORY OF THIS FIRMWARE PACKAGE.
  *
  * <h2><center>&copy; COPYRIGHT 2012 STMicroelectronics</center></h2>
  ******************************************************************************
  */

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __AUDIO_APP_CONF__H__
#define __AUDIO_APP_CONF__H__

/* Includes ------------------------------------------------------------------*/
#ifdef STM32F2XX
 #include "stm32f2xx.h"
#elif defined(STM32F10X_CL)
 #include "stm32f10x.h"
#elif defined(STM32F4XX)
 #include "stm32f4xx.h"
#endif /* STM32F2XX */

/** @defgroup AUDIO_APP_CONF_Exported_Defines
  * @{
  */ 

/* Uncomment this define to enable the use of SRC synchronization.
   Thid define may be set at the project configuration level. */
/* #define AUDIO_SYNCHRO_SRC_ENABLED                1 */

/*-----------------------------------------------------------------------------
   Defines from usbd_conf.h file 
------------------------------------------------------------------------------*/
/* Set this define to 1 to enable the USB audio IN flow (microphone)
   or set it to 0 to disable the USB audio IN flow. */
#ifdef USE_STM3210C_EVAL
 #define AUDIO_IN_ENABLED                          0
#else
 #define AUDIO_IN_ENABLED                          1
#endif /* USE_STM3210C_EVAL */

/* Default audio frequency in Hz. If SUPPORTED_FREQ_NBR > 1, make sure that the default frequency is
   always the higher one in all supported frequencies (because the allocation of data buffer is done 
   based on this frequency). */
#define DEFAULT_OUT_AUDIO_FREQ                     48000 

/* Number of supported audio frequencies (Min = 1, Max = 4).
   Note: when supported number of frequencies is > 1 and the Feedback synchronization method is
   used (AUDIO_SYNCHRO_FEEDBACK_ENABLED) then the behavior of the host may be unpredictable:
   In some cases, the host may use the dynamic audio freq switching while playing file which
   may lead (when switching frequency is too high) to glitches on audio output (due to delay
   caused by the dynamic clock switching). */
#define SUPPORTED_FREQ_NBR                         1

/* List of supported audio frequencies. You can only change order between freq 2, 3 and 4.
   To add new frequencies, you have to do some modifications on stm322xg_usb_audio_codec.c 
   and stm322xg_clk_synthesizer_cs2200.c files.*/
#define USBD_AUDIO_FREQ_1                          DEFAULT_OUT_AUDIO_FREQ /* Audio frequency in Hz.*/
#define USBD_AUDIO_FREQ_2                          44100 /* Audio frequency in Hz.*/
#define USBD_AUDIO_FREQ_3                          32000 /* Audio frequency in Hz.*/
#define USBD_AUDIO_FREQ_4                          44000 /* Audio frequency in Hz. This freq is used for debug purpose. */

#define DEFAULT_VOLUME                             80    /* Default volume in % (Mute=0%, Max = 100%) in Logarithmic values.
                                                 To get accurate volume variations, it is possible to use a logarithmic
                                                 conversion table to convert from percentage to logarithmic law.
                                                 In order to keep this example code simple, this conversion is not used.*/

/* Default audio frequency in Hz. If SUPPORTED_FREQ_NBR > 1, make sure that the default frequency is
   always the higher one in all supported frequencies (because the allocation of data buffer is done 
   based on this frequency). */
#define DEFAULT_IN_AUDIO_FREQ                      8000 

/*-----------------------------------------------------------------------------
   Defines from usbd_audio_synchprocess.h file 
------------------------------------------------------------------------------*/
/* Set the total number of methods to be used in the application.
   This number depends on the number of defines enabled among the following:
       AUDIO_SYNCHRO_ADDREM_ENABLED
       AUDIO_SYNCHRO_CLKSYNTH_ENABLED
       AUDIO_SYNCHRO_FEEDBACK_ENABLED
       AUDIO_SYNCHRO_SRC_ENABLED
   Please note that the SYNCHRO_NONE mehtod is always enabled (maximum number
   of synchronization methods = 5, minimum number of methods = 1)
     
     */
#ifdef USE_STM3210C_EVAL
 #define NBR_METHODS                               3 /* None, FeedBack, AddRem */
#elif defined(USE_STM32F4XX_DISCOVERY)
 #define NBR_METHODS                               4 /* None, FeedBack, AddRem, SRC */
#elif defined(USE_STM322xG_EVAL) 
 #define NBR_METHODS                               5 /* None, ExtClk, FeedBack, AddRem, SRC */
#elif defined(USE_STM324xG_EVAL) 
 #define NBR_METHODS                               5 /* None, ExtClk, FeedBack, AddRem, SRC */
#endif /* USE_STM3210C_EVAL,  */


/* Set the relative define to 0 to disable a synchronization method or to 1 to 
   enable that method.
   You can disable all methods: In this case no correction method is selected
   and the streaming will not be synchronized. 

   Add-Remove samples synchronization method. This method doesn't require 
   additional hardware. */
#define AUDIO_SYNCHRO_ADDREM_ENABLED               1

 /* External Dynamic PLL with clock synthesizer. This method uses the clock 
    synthesizer component implemented in STM322xG-EVAL revB board. */
#if defined(USE_STM3210C_EVAL) || defined(USE_STM32F4XX_DISCOVERY)
 #define AUDIO_SYNCHRO_CLKSYNTH_ENABLED            0
#else
 #define AUDIO_SYNCHRO_CLKSYNTH_ENABLED            1
#endif /* USE_STM322xG_EVAL */

/* Feedback pipe synchronization method. This method doesn't require 
   additional hardware. */
#define AUDIO_SYNCHRO_FEEDBACK_ENABLED             1

 /* Dynamic Audio recording clock adjustment using Timer auto reload register udpate. */
#if defined(USE_STM3210C_EVAL) || defined(USE_STM32F4XX_DISCOVERY)
 #define USB_AUDIO_IN_SYNCHRO_CLKUPDT_ENABLED      0
#else
 #define USB_AUDIO_IN_SYNCHRO_CLKUPDT_ENABLED      1
#endif /* USE_STM322xG_EVAL */



/* Correction timing parameters: Make sure these parameters are set to suitable 
  values depending on the allowed error and the application requirements. */
/*-------------------- SYNCH_NONE -------------------------------------------*/
#define SYNCH_NONE_PERIOD                         300 /* Used as indicative way only */
#define SYNCH_NONE_STEP                           0   /* Not used in this case */

/*-------------------- SYNCH_EXTCLK -----------------------------------------*/
/* Period of time between corrections in ms (This value must be multiple of 10 ms).
    This constraint (multiple of 10 ms) is only to avoid intensive entrance into 
    interrupt routine (save CPU load). If higher granularity is required you can 
    disable the option AUDIO_CORR_INERR_MULT_10 by commenting the define. */
/* This is the case when clock shift is less than 100ppm */
#define SYNCH_EXTCLK_PERIOD                       200
/* Step for ppm adjustment */
#define SYNCH_EXTCLK_STEP                         1
/* This is the case when clock shift is higher than 100ppm and less than 500ppm */
#define SYNCH_EXTCLK_PERIOD_ALT                   200
/* Step for ppm adjustment */
#define SYNCH_EXTCLK_STEP_ALT                     5
/* External Clock synchro threshold not used here, keep it at 0 */
#define SYNCH_EXTCLK_THRESHOLD                    0


/*-------------------- SYNCH_FEEDBACK ---------------------------------------*/
#ifndef EXTERNAL_CRYSTAL_25MHz
 /* Period of time between corrections in ms (This value must be multiple of 10 ms).
    This constraint (multiple of 10 ms) is only to avoid intensive entrance into 
    interrupt routine (save CPU load). If higher granularity is required you can 
    disable the option AUDIO_CORR_INERR_MULT_10 by commenting the define. */
 #define SYNCH_FEEDBACK_PERIOD                    500
 /* Fractional part to be added/removed to/from original frame size (fractional
    format is 10.10 (or 10.14), refer to USB 2.0 specification for more details).
    when the define FEEDBACK_EXTENDED_PRECISION_10_14 is enabled, the used format
    is 10.14. Else it is 10.10 */
 #define SYNCH_FEEDBACK_STEP                      1
 /* Feedback pipe refresh value. This value indicates the rate at which the host
    should request for the updated feedback value. Min = 1, Max = 9.
    The refresh period = 2^FEEDBACK_REFRESH. 
    e.g. FEEDBACK_REFRESH = 5: bRefresh = 2^5 = 32 */
 #define FEEDBACK_REFRESH                         8
 /* Feedback threshold is the maximum number of corrections in one direction.
    This parameter defines the dynamic of the feedback value update.
    e.g if FEEDBACK_THRESHOLD = 30: if the feedback value is decremented 30 times
    then it is reset to the orginal value. 
    You can set this value to 0xFFFFFFF to see the effect of not limiting the 
    dynamics of feedback by a threshold value: correction will go too far in 
    one direction before going in the opposite one. */
 #define SYNCH_FEEDBACK_THRESHOLD                 30

#else /* #ifdef EXTERNAL_CRYSTAL_25MHz */
 /* Period of time between corrections in ms (This value must be multiple of 10 ms).
    This constraint (multiple of 10 ms) is only to avoid intensive entrance into 
    interrupt routine (save CPU load). If higher granularity is required you can 
    disable the option AUDIO_CORR_INERR_MULT_10 by commenting the define. */
 #define SYNCH_FEEDBACK_PERIOD                    50
 /* Fractional part to be added/removed to/from original frame size (fractional
    format is 10.10 (or 10.14), refer to USB 2.0 specification for more details).
    when the define FEEDBACK_EXTENDED_PRECISION_10_14 is enabled, the used format
    is 10.14. Else it is 10.10 */
 #define SYNCH_FEEDBACK_STEP                      20
 /* Feedback pipe refresh value. This value indicates the rate at which the host
    should request for the updated feedback value. Min = 1, Max = 9.
    The refresh period = 2^FEEDBACK_REFRESH. 
    e.g. FEEDBACK_REFRESH = 5: bRefresh = 2^5 = 32 */
 #define FEEDBACK_REFRESH                         4
 /* Feedback threshold is the maximum number of corrections in one direction.
    This parameter defines the dynamic of the feedback value update.
    e.g if FEEDBACK_THRESHOLD = 30: if the feedback value is decremented 30 times
    then it is reset to the orginal value. 
    You can set this value to 0xFFFFFFF to see the effect of not limiting the 
    dynamics of feedback by a threshold value: correction will go too far in 
    one direction before going in the opposite one. */
 #define SYNCH_FEEDBACK_THRESHOLD                 30
#endif /* EXTERNAL_CRYSTAL_25MHz,  */

/*-------------------- SYNCH_ADDREM -----------------------------------------*/
 /* Period of time between corrections in ms (This value must be multiple of 10 ms).
    This constraint (multiple of 10 ms) is only to avoid intensive entrance into 
    interrupt routine (save CPU load). If higher granularity is required you can 
    disable the option AUDIO_CORR_INERR_MULT_10 by commenting the define. */
#define SYNCH_ADDREM_PERIOD                       50
 /* Number of samples to be removed or added each time an overrun/underrun 
    condition is detected. */
#define SYNCH_ADDREM_STEP                         4

/*-------------------- SYNCH_SRC --------------------------------------------*/
 /* Period of time between corrections in ms (This value must be multiple of 10 ms).
    This constraint (multiple of 10 ms) is only to avoid intensive entrance into 
    interrupt routine (save CPU load). If higher granularity is required you can 
    disable the option AUDIO_CORR_INERR_MULT_10 by commenting the define. */
#define SYNCH_SRC_PERIOD                          300//200
 /* Number of samples to be removed or added each time an overrun/underrun 
    condition is detected. */
#define SYNCH_SRC_STEP                            4

/* Selec one of the above SRC Synchronization profiles */
#define SRC_SYNCHRO_PROFILE_1
//#define SRC_SYNCHRO_PROFILE_2
//#define SRC_SYNCHRO_PROFILE_3

/* Resampling ratio ratio = 44.1/48 * 2^28  = Fin/Fout*2^28 */
#define RATIO_1                                     268435456 
#define RATIO_48_TO_48_01                           268379544
#define RATIO_48_TO_47_99                           268491392

/* SRC Synchronization profile definitions */
#ifdef SRC_SYNCHRO_PROFILE_1
  /* Profile 1: Low Dynamic ----------------------------------*/
  #define SRC_SYNC_PERIOD_MAX                       600
  #define SRC_SYNC_PERIOD_MIN                       100
  #define SRC_SYNC_PERIOD_UP                        50
  #define SRC_SYNC_PERIOD_DOWN                      10
  #define SRC_SYNC_FACTOR_MAX                       8
  #define SRC_SYNC_FACTOR_MIN                       1
  #define SRC_SYNC_FACTOR_HIGH                      4
  #define SRC_SYNC_FACTOR_LOW                       2
  
  #define SRC_RATIO_STEP_0_001                      (5592) /* Equivalent to 0.001 KHz for 48KHz*/ //560 equivalent to 0.0001 KHz
  #define SRC_RATIO_MAX                             RATIO_48_TO_47_99 //RATIO_48_TO_47_98
  #define SRC_RATIO_MIN                             RATIO_48_TO_48_01 //RATIO_48_TO_48_02

#elif defined (SRC_SYNCHRO_PROFILE_2)
  /* Profile 2: High Dynamic ----------------------------------*/
  #define SRC_SYNC_PERIOD_MAX                       600
  #define SRC_SYNC_PERIOD_MIN                       50
  #define SRC_SYNC_PERIOD_UP                        30
  #define SRC_SYNC_PERIOD_DOWN                      10
  #define SRC_SYNC_FACTOR_MAX                       8
  #define SRC_SYNC_FACTOR_MIN                       1
  #define SRC_SYNC_FACTOR_HIGH                      4
  #define SRC_SYNC_FACTOR_LOW                       2
  
  #define SRC_RATIO_STEP_0_001                      (5592) /* Equivalent to 0.001 KHz for 48KHz*/ //560 equivalent to 0.0001 KHz
  #define SRC_RATIO_MAX                             RATIO_48_TO_47_99 //RATIO_48_TO_47_98
  #define SRC_RATIO_MIN                             RATIO_48_TO_48_01 //RATIO_48_TO_48_02

#elif defined (SRC_SYNCHRO_PROFILE_3)
  /* Profile 1: Very Low Dynamic -------------------------------*/
  #define SRC_SYNC_PERIOD_MAX                       1000
  #define SRC_SYNC_PERIOD_MIN                       200
  #define SRC_SYNC_PERIOD_UP                        40//50
  #define SRC_SYNC_PERIOD_DOWN                      10
  #define SRC_SYNC_FACTOR_MAX                       2//256
  #define SRC_SYNC_FACTOR_MIN                       1
  #define SRC_SYNC_FACTOR_HIGH                      4
  #define SRC_SYNC_FACTOR_LOW                       2
  
  #define SRC_RATIO_STEP_0_001                      (5592) /* Equivalent to 0.001 KHz for 48KHz*/ //560 equivalent to 0.0001 KHz
  #define SRC_RATIO_MAX                             RATIO_48_TO_47_99 //RATIO_48_TO_47_98
  #define SRC_RATIO_MIN                             RATIO_48_TO_48_01 //RATIO_48_TO_48_02

#else
 #error "Please select one SRC Synchro profile in audio_app_conf.h"
#endif /*  */

#define RATIO_UP                                  RATIO_48_TO_48_0010
#define RATIO_DOWN                                268463421 //RATIO_48_TO_47_9950

#define SRC_BLOCK_THRESHOLD                       10
#define SRC_MAX_BLOCK_TIME                        1000


/* Number of frames necessary to achieve adding/removing 1 sample
  ie. if sampling rates are 48 - 48.05 - 47.95 KHz then this number 
  is equal to 20: 20 frames x 0.05 = 1 sample */
#define SRC_COUNT_DONE                            50 

/* Total available ratios number */
//#define SRC_NUMBER_RATIOS                         5                 

/*-------------------- SYNCH_CLKUPDT ----------------------------------------*/
 /* Period of time between corrections in ms (This value must be multiple of 10 ms).
    This constraint (multiple of 10 ms) is only to avoid intensive entrance into 
    interrupt routine (save CPU load). If higher granularity is required you can 
    disable the option AUDIO_CORR_INERR_MULT_10 by commenting the define. */
 #define SYNCH_CLKUPDT_PERIOD                      50 /* 50 ms */
 /* Step value for the input clock update. */
 #define SYNCH_CLKUPDT_STEP                        2

/*-------------------- Optional adjustment Parameters -----------------------*/
/* If higher granularity is required for CORRECTION_PERIOD you can disable the option
    AUDIO_CORR_INERR_MULT_10 by commenting this define.
    In this case the CORRECTION_PERIOD value can be non multiple of 10.
    WARNING: Uncommenting this define will lead to increasing CPU load since the periodic
    correction interrupt will be exected at higher frequency. */
#define AUDIO_CORR_INERR_MULT_10

/* Uncomment this define to extend the precision of the feedback value from 10.10 to 10.14
   format. This means that the lower 4 bits of the fractional part will be used.*/
#define FEEDBACK_EXTENDED_PRECISION_10_14

/* Uncomment these defines to enable the Dynamic Adjustment Factor: that means
   the correction step will be increased/decreased depending on the error value.
   This method is available only with external clock synthesizer correction enabled. */
#define USE_DYNAMIC_ADJUSTMENT_FACTOR              0
#define DYNAMIC_ADJUSTMENT_FACTOR_MAX              4


/* Number of sub-packets in the audio transfer buffer. You can modify this value but always make sure
  that it is an even number and higher than 3 */
#define OUT_PACKET_NUM                             20//500


/* Total size of the audio Out transfer buffer */
#define OUT_BUF_MARGIN                             4 /* Margin for each sub-buffer */
#define TOTAL_OUT_BUF_SIZE                         ((uint32_t)((AUDIO_OUT_PACKET + OUT_BUF_MARGIN) * OUT_PACKET_NUM))

/* Number of sub-packets in the audio transfer buffer. You can modify this value but always make sure
  that it is an even number and higher than 3 */
#define IN_PACKET_NUM                              20

/* Total size of the audio In transfer buffer */
#define IN_BUF_MARGIN                              4 /* Margin for each sub-buffer */
#define TOTAL_IN_BUF_SIZE                          ((uint32_t)((AUDIO_IN_PACKET + IN_BUF_MARGIN) * IN_PACKET_NUM))


/*
  Interrupt's priorities configuration:
  All application interrupt's priorities configuration are gathered in this section
  to make it easier to control and update.
   - Interrupt priority grouping is set to 2: 2 bits for Preemption priority and 2 bits for Sub-Priority
   - Select the preemption priority level(0 is the highest, 3 is the lowest)
   - Select the sub-priority level (0 is the highest, 3 is the lowest) 
  Make sure you configure correctly the interrupt priorities
  because bad configuration may easilly lead to application not
  working or not working correctly.
*/
/*-----------------------------------------------------------------------------
   Defines from stm322xg_audio_codec.h, stm32f4_discovery_audio_codec.h and
   stm3210c_audio_codec.h files 
------------------------------------------------------------------------------*/
//#define EVAL_AUDIO_IRQ_PREPRIO                     1 
//#define EVAL_AUDIO_IRQ_SUBRIO                      1 
#define EVAL_AUDIO_IRQ_PREPRIO                     0 
#define EVAL_AUDIO_IRQ_SUBRIO                      0 

/*-----------------------------------------------------------------------------
   Defines from stm32_audio_recorder.h file 
------------------------------------------------------------------------------*/
#define AUDIO_REC_TIM_PRE_PRIO                     1
#define AUDIO_REC_TIM_SUB_PRIO                     0

#define AUDIO_REC_SPI_PRE_PRIO                     3
#define AUDIO_REC_SPI_SUB_PRIO                     1

/*-----------------------------------------------------------------------------
   Defines from usbd_audio_correction.h file 
------------------------------------------------------------------------------*/
#define AUDIO_CORR_TIM_PRE_PRIO                    3
#define AUDIO_CORR_TIM_SUB_PRIO                    1

/*-----------------------------------------------------------------------------
   Defines from usbd_audio_correction.h file 
------------------------------------------------------------------------------*/
//#define SRC_IRQ_PREPRIO                            0
//#define SRC_IRQ_SUBPRIO                            0
//#define SRC_IRQ_PREPRIO                            1
//#define SRC_IRQ_SUBPRIO                            1

/*-----------------------------------------------------------------------------
   Defines from usbd_conf.h file 
------------------------------------------------------------------------------*/
#define USB_OTG_IRQ_PREPRIO                        1
#define USB_OTG_IRQ_SUBPRIO                        3
//#define USB_OTG_IRQ_PREPRIO                        1
//#define USB_OTG_IRQ_SUBPRIO                        0


/*-----------------------------------------------------------------------------
   Defines from stm322xg_clk_synthesizer_cs2200.h file 
------------------------------------------------------------------------------*/
#define STM32_CLK_SYNTH_PRE_PRIO                   3
#define STM32_CLK_SYNTH_SUB_PRIO                   3

/*-----------------------------------------------------------------------------
   Defines from usbd_conf.h file 
------------------------------------------------------------------------------*/
#define AUDIO_OUT_BUFF_XFER_CPLT                   usbd_audio_BuffXferCplt
extern uint8_t  usbd_audio_BuffXferCplt (uint8_t Direction, uint8_t** pbuf, uint32_t* pSize);


#if defined(BENCHMARK_ENABLED)
#define BENCH_SIZE                                  1000
#endif /* BENCHMARK_ENABLED */


#if defined(USE_STM32F4XX_DISCOVERY)
 #define LED1                                       LED5
 #define LED2                                       LED6
#endif /* USE_STM32F4XX_DISCOVERY */

/**
  * @}
  */ 





/** @defgroup AUDIO_APP_CONF_Exported_Types
  * @{
  */ 
/**
  * @}
  */ 


/** @defgroup AUDIO_APP_CONF_Exported_Macros
  * @{
  */ 
/**
  * @}
  */ 

/** @defgroup AUDIO_APP_CONF_Exported_Variables
  * @{
  */ 
/**
  * @}
  */ 

/** @defgroup AUDIO_APP_CONF_Exported_FunctionsPrototype
  * @{
  */ 
/**
  * @}
  */ 


#endif /* __AUDIO_APP_CONF__H__ */

/******************* (C) COPYRIGHT 2012 STMicroelectronics *****END OF FILE****/

