/**
  ******************************************************************************
  * @file    usbd_conf.h
  * @author  MCD Application Team
  * @version V1.0.0RC2
  * @date    13-February-2012
  * @brief   USB Device configuration file
  ******************************************************************************
  * @attention
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * FOR MORE INFORMATION PLEASE READ CAREFULLY THE LICENSE AGREEMENT FILE
  * LOCATED IN THE ROOT DIRECTORY OF THIS FIRMWARE PACKAGE.
  *
  * <h2><center>&copy; COPYRIGHT 2012 STMicroelectronics</center></h2>
  ******************************************************************************
  */

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __USBD_CONF__H__
#define __USBD_CONF__H__

/* Includes ------------------------------------------------------------------*/
#if defined (USE_STM322xG_EVAL)
 #include "stm322xg_eval.h"
#elif defined(USE_STM324xG_EVAL)
 #include "stm324xg_eval.h"
#elif defined (USE_STM32F4XX_DISCOVERY)
 #include  "stm32f4_discovery.h"
#elif defined (USE_STM3210C_EVAL)
 #include "stm3210c_eval.h"
#else
 #error "Missing define: Evaluation board (ie. USE_STM322xG_EVAL)"
#endif /* USE_STM32F4XX_DISCOVERY */

#include "audio_app_conf.h"

/** @defgroup USB_CONF_Exported_Defines
  * @{
  */ 

/* ----------------------------------------------------------------------------

   All the commented configuration parameters are gathered into audio_app_conf.h 
   file. If you already set them in audio_app_conf.h file, there is no need 
   to reconfigure them here.

-----------------------------------------------------------------------------*/

/* Set this define to 1 to enable the USB audio IN flow (microphone)
   or set it to 0 to disable the USB audio IN flow. */
/* #define AUDIO_IN_ENABLED                1 */

/* Default audio frequency in Hz. If SUPPORTED_FREQ_NBR > 1, make sure that the default frequency is
   always the higher one in all supported frequencies (because the allocation of data buffer is done 
   based on this frequency). */
/* #define DEFAULT_OUT_AUDIO_FREQ          48000 */

/* Number of supported audio frequencies (Min = 1, Max = 4).
   Note: when supported number of frequencies is > 1 and the Feedback synchronization method is
   used (AUDIO_SYNCHRO_FEEDBACK_ENABLED) then the behavior of the host may be unpredictable:
   In some cases, the host may use the dynamic audio freq switching while playing file which
   may lead (when switching frequency is too high) to glitches on audio output (due to delay
   caused by the dynamic clock switching). */
/* #define SUPPORTED_FREQ_NBR              1 */

/* List of supported audio frequencies. You can only change order between freq 2, 3 and 4.
   To add new frequencies, you have to do some modifications on stm322xg_usb_audio_codec.c 
   and stm322xg_clk_synthesizer_cs2200.c files.*/
/* #define USBD_AUDIO_FREQ_1               DEFAULT_OUT_AUDIO_FREQ *//* Audio frequency in Hz.*/
/* #define USBD_AUDIO_FREQ_2               44100 *//* Audio frequency in Hz.*/
/* #define USBD_AUDIO_FREQ_3               32000 *//* Audio frequency in Hz.*/
/* #define USBD_AUDIO_FREQ_4               44000 *//* Audio frequency in Hz. This freq is used for debug purpose. */

/* #define DEFAULT_VOLUME                  80    *//* Default volume in % (Mute=0%, Max = 100%) in Logarithmic values.
                                                 To get accurate volume variations, it is possible to use a logarithmic
                                                 conversion table to convert from percentage to logarithmic law.
                                                 In order to keep this example code simple, this conversion is not used.*/


/* Default audio frequency in Hz. If SUPPORTED_FREQ_NBR > 1, make sure that the default frequency is
   always the higher one in all supported frequencies (because the allocation of data buffer is done 
   based on this frequency). */
/* #define DEFAULT_IN_AUDIO_FREQ           8000 */


/* Use this section to modify the number of supported interfaces and configurations.
   Note that if you modify these parameters, you have to modify the descriptors 
   accordingly in usbd_audio_core.c file */
#define AUDIO_TOTAL_IF_NUM              0x04
#define USBD_CFG_MAX_NUM                1
#define USBD_ITF_MAX_NUM                3
#define USB_MAX_STR_DESC_SIZ            200 


/** @defgroup USB_AUDIO_Class_Layer_Parameter
  * @{
  */ 
#define AUDIO_OUT_EP                    0x01 /* Audio Data out endpoint */
#define AUDIO_FB_EP                     0x81 /* Audio explicit Feedback endpoint */
#define AUDIO_IN_EP                     0x82 /* Audio Data in endpoint */


/* #define USB_OTG_IRQ_PREPRIO                       1 */
/* #define USB_OTG_IRQ_SUBPRIO                       3 */

/**
  * @}
  */ 

/** @defgroup USB_CONF_Exported_Types
  * @{
  */ 
/**
  * @}
  */ 


/** @defgroup USB_CONF_Exported_Macros
  * @{
  */ 
/**
  * @}
  */ 

/** @defgroup USB_CONF_Exported_Variables
  * @{
  */ 
/**
  * @}
  */ 

/** @defgroup USB_CONF_Exported_FunctionsPrototype
  * @{
  */ 
/**
  * @}
  */ 


#endif /* __USBD_CONF__H__ */

/******************* (C) COPYRIGHT 2012 STMicroelectronics *****END OF FILE****/

