/**
  ******************************************************************************
  * @file    app.c
  * @author  MCD Application Team
  * @version V1.0.0RC2
  * @date    13-February-2012
  * @brief   This file provides all the Application firmware functions.
  ******************************************************************************
  * @attention
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * FOR MORE INFORMATION PLEASE READ CAREFULLY THE LICENSE AGREEMENT FILE
  * LOCATED IN THE ROOT DIRECTORY OF THIS FIRMWARE PACKAGE.
  *
  * <h2><center>&copy; COPYRIGHT 2012 STMicroelectronics</center></h2>
  ******************************************************************************
  */ 

/* Includes ------------------------------------------------------------------*/ 

#include  "usbd_usr.h"
#include  "usbd_audio_core.h"

#if defined (USE_STM322xG_EVAL)
 #include "stm322xg_eval.h"
#elif defined(USE_STM324xG_EVAL)
 #include "stm324xg_eval.h"
#elif defined (USE_STM32F4XX_DISCOVERY)
 #include  "stm32f4_discovery.h"
#elif defined (USE_STM3210C_EVAL)
 #include "stm3210c_eval.h"
#else
 #error "Missing define: Evaluation board (ie. USE_STM322xG_EVAL)"
#endif /* USE_STM32F4XX_DISCOVERY */

#include  "stm32_audio.h"

#if !defined(NO_LCD)
 #include "lcd_log.h"
#endif /* USE_STM32F4XX_DISCOVERY */

#if defined(BENCHMARK_ENABLED)
 #include "stm32_audio_out_if.h"
#endif /* BENCHMARK_ENABLED */

/** @addtogroup STM32_USB_OTG_DEVICE_LIBRARY
  * @{
  */


/** @defgroup APP_AUDIO 
  * @brief Mass storage application module
  * @{
  */ 

/** @defgroup APP_AUDIO_Private_TypesDefinitions
  * @{
  */ 
/**
  * @}
  */ 


/** @defgroup APP_AUDIO_Private_Defines
  * @{
  */ 
#define LINE_01                           (LCD_PIXEL_HEIGHT - 60)
#define LINE_02                           (LCD_PIXEL_HEIGHT - 48)
#define LINE_03                           (LCD_PIXEL_HEIGHT - 36) 
#define LINE_04                           (LCD_PIXEL_HEIGHT - 24) 
#define LINE_05                           (LCD_PIXEL_HEIGHT - 12)  

/**
  * @}
  */ 


/** @defgroup APP_AUDIO_Private_Macros
  * @{
  */ 
/**
  * @}
  */ 


/** @defgroup APP_AUDIO_Private_Variables
  * @{
  */ 
  
#ifdef USB_OTG_HS_INTERNAL_DMA_ENABLED
#pragma data_alignment = 4 
#endif
USB_OTG_CORE_HANDLE           USB_OTG_dev;

SynchMethod_TypeDef tSynchMethod[] = {
  SYNCH_NONE,      /* OUT/IN: No Synchronization */
 #if (AUDIO_SYNCHRO_CLKSYNTH_ENABLED == 1)
  SYNCH_EXTCLK,    /* OUT: Extrenal Clock Synthesizer synchroization */
 #endif
 #if (AUDIO_SYNCHRO_FEEDBACK_ENABLED == 1)
  SYNCH_FEEDBACK,  /* OUT: Feedback isoc pipe synchronization  */
 #endif
 #if (AUDIO_SYNCHRO_ADDREM_ENABLED == 1)
  SYNCH_ADDREM,    /* OUT: Add/Remove samples synchronization */  
 #endif
 #ifdef AUDIO_SYNCHRO_SRC_ENABLED
  SYNCH_SRC,       /* OUT: SRC synchronization */  
 #endif /* AUDIO_SYNCHRO_SRC_ENABLED */
};

uint8_t * DEMO_main_menu[] = 
{
  "       - No Synchro                   ",
 #if (AUDIO_SYNCHRO_CLKSYNTH_ENABLED == 1)
  "       - Extern Clock Synchro         ",
 #endif
 #if (AUDIO_SYNCHRO_FEEDBACK_ENABLED == 1)
  "       - Feedback Pipe Synchro        ",
 #endif
 #if (AUDIO_SYNCHRO_ADDREM_ENABLED == 1)
  "       - Add-Remove samples Synchro   ",
 #endif
 #ifdef AUDIO_SYNCHRO_SRC_ENABLED
  "       - Source Rate Converter        ",
 #endif /* AUDIO_SYNCHRO_SRC_ENABLED*/
};

uint8_t* tMethods [] = {
  "USB Audio -  Synchro None   \n",
 #if (AUDIO_SYNCHRO_CLKSYNTH_ENABLED == 1)
  "USB Audio -  Extern Clock   \n",
 #endif
 #if (AUDIO_SYNCHRO_FEEDBACK_ENABLED == 1)
  "USB Audio -  Feedback Pipe  \n",
 #endif
 #if (AUDIO_SYNCHRO_ADDREM_ENABLED == 1)
  "USB Audio -  Add-Remove     \n",
 #endif
 #if (AUDIO_SYNCHRO_SRC_ENABLED == 1)
  "USB Audio -  SRC            \n",
 #endif /* AUDIO_SYNCHRO_SRC_ENABLED */
};

#ifndef NO_LCD
uint8_t tLinesTab[] = {
  LINE_01,
  LINE_02,
  LINE_03,
  LINE_04,
  LINE_05,
};
#endif /* NO_LCD */

uint8_t CurrItem = 0;
uint8_t Exit = 0;

#if defined(BENCHMARK_ENABLED)
__IO uint32_t tBenchTable[BENCH_SIZE];
__IO uint32_t BenchCount = 0;
__IO uint32_t BenchState = 0;
__IO uint32_t BenchRef = 0;
__IO uint32_t Load = 0;
static uint32_t count = 0;
#endif /* BENCHMARK_ENABLED */
/**
  * @}
  */ 


/** @defgroup APP_AUDIO_Private_FunctionPrototypes
  * @{
  */ 
extern void usbd_audio_DescUpdate(uint32_t FBstate);
#ifndef NO_LCD
static void Demo_SelectItem (uint8_t **menu , uint8_t item, uint8_t size);
#endif /* NO_LCD */
/**
  * @}
  */ 


/** @defgroup APP_AUDIO_Private_Functions
  * @{
  */ 

/**
  * @brief  Program entry point
  * @param  None
  * @retval None
  */
int main(void)
{
  /*!< At this stage the microcontroller clock setting is already configured, 
  this is done through SystemInit() function which is called from startup
  file (startup_stm32fxxx_xx.s) before to branch to application main.
  To reconfigure the default setting of SystemInit() function, refer to
  system_stm32fxxx.c file
  */

#if defined(BENCHMARK_ENABLED)
  /* Setup SysTick Timer for 10 msec interrupts 
     This interrupt is used to display the current state of the Audio Player and
     the output state (Speaker/Headphone) */
  if (SysTick_Config(SystemCoreClock / 100))
  { 
    /* Capture error */ 
    while (1);
  }
  NVIC_SetPriority (SysTick_IRQn, 0);

  /* Start benchmarking */
  BenchState = 1;

  /* Benchmark loop */
  while (BenchState == 1)
  {  
    BenchCount++;
  }

  /* Get the benchmark reference (CPU Idle) */
  BenchRef = BenchCount;

  /* Reset the benchmark counter */
  BenchCount = 0;

#endif /* BENCHMARK_ENABLED */

  /* Display menu */
#ifdef NO_LCD
  /* Set the synhcronization methods for Out and In streams */
  AUDIO_CORR_SetActiveMethod(DIRECTION_OUT, SYNCH_SRC); 
  AUDIO_CORR_SetActiveMethod(DIRECTION_IN, SYNCH_NONE);
  
  /* Update the descriptor to support the feedback pipe */
  usbd_audio_DescUpdate(SYNCH_SRC);

#else
  /* Configure control buttons */
  STM_EVAL_PBInit(Button_KEY, Mode_GPIO);
  STM_EVAL_PBInit(Button_TAMPER, Mode_GPIO);
  STM_EVAL_PBInit(Button_WAKEUP, Mode_GPIO);

  /* Initialize the LCD */
#ifdef USE_STM3210C_EVAL
  STM3210C_LCD_Init();  
#elif defined (USE_STM322xG_EVAL)
  STM322xG_LCD_Init();
#elif defined(USE_STM324xG_EVAL)
  STM324xG_LCD_Init();
#endif

  LCD_LOG_Init();
  
  LCD_LOG_SetHeader("USB Audio Selection Menu ");
  LCD_UsrLog("Use Key, Tamper and Wakeup push buttons to select a program: \n");
  LCD_UsrLog("> Up/Down :   Wakeup/Tamper\n");
  LCD_UsrLog("> SELECT  :   Key          \n");
  Demo_SelectItem (DEMO_main_menu , 0, NBR_METHODS);
  
  /* Loop until Key push button is pushed */
  while (Exit == 0)
  {
    /* Key Push button: Select */
    if (STM_EVAL_PBGetState(Button_KEY) == 0x00)
    {
      /* Avoid rebound */
      while (STM_EVAL_PBGetState(Button_KEY) == 0x00)
      {}
      
      /* Set the synhcronization methods for Out and In streams */
      AUDIO_CORR_SetActiveMethod(DIRECTION_OUT, tSynchMethod[CurrItem]); 
      AUDIO_CORR_SetActiveMethod(DIRECTION_IN, SYNCH_NONE /*SYNCH_CLKUPDT*/);
      
      /* Update the descriptor to support the feedback pipe */
      usbd_audio_DescUpdate(tSynchMethod[CurrItem]);  
      
      /* Prepare exiting from loop */
      Exit = 1;
    }
    /* Tamper push button: Down */
    else if (STM_EVAL_PBGetState(Button_TAMPER) == 0x00)
    {
      /* Avoid rebound */
      while(STM_EVAL_PBGetState(Button_TAMPER) == 0x00)
      {}
      
      if (CurrItem < (NBR_METHODS - 1))
      {
        CurrItem++;
      }
      else
      {
        CurrItem = 0;
      }
      
      /* Update the main menu selection */
      Demo_SelectItem (DEMO_main_menu , CurrItem, NBR_METHODS);
    }
    /* Wakeup push button: Up */
    else if (STM_EVAL_PBGetState(Button_WAKEUP) != 0x00)
    {
      /* Avoid rebound */
      while (STM_EVAL_PBGetState(Button_WAKEUP) != 0x00)
      {}
      
      if (CurrItem == 0)
      {
        CurrItem = (NBR_METHODS - 1);
      }
      else
      {
        CurrItem--;
      }
      
      /* Update the main menu selection */
      Demo_SelectItem (DEMO_main_menu , CurrItem, NBR_METHODS);
    }
  }
  
  /* Clear LCD menu */
  LCD_LOG_ClearTextZone(); 
#endif /* NO_LCD */  
  
  USBD_Init(&USB_OTG_dev,
#ifdef USE_USB_OTG_HS 
            USB_OTG_HS_CORE_ID,
#else            
            USB_OTG_FS_CORE_ID,
#endif            
            &USR_desc, 
            &AUDIO_cb, 
            &USR_cb);


  /* Setup SysTick Timer for 100 msec interrupts 
     This interrupt is used to display the current state of the Audio Player and
     the output state (Speaker/Headphone) */
#if !defined(NO_LCD) && !defined(BENCHMARK_ENABLED)
  if (SysTick_Config(SystemCoreClock / 10))
  { 
    /* Capture error */ 
    while (1);
  }
#endif /* NO_LCD */ 

  /* Main loop */
  while (1)
  {  
#if defined(BENCHMARK_ENABLED)
    /* Set SysTick priority to highest */
    NVIC_SetPriority (SysTick_IRQn, 0);
    
    /* Wait for player to start */
    while (AUDIO_OUT_fops.GetState() != AUDIO_STATE_PLAYING)
    {}
    
    while (count++ < BENCH_SIZE)
    {
      /* Start benchmarking */
      BenchState = 1;
      
      /* Benchmark loop */
      while (BenchState == 1)
      {  
        BenchCount++;
      }
      
      /* Get the benchmark reference (CPU Idle) */
      tBenchTable[count] = BenchCount;
      
      /* Reset the benchmark counter */
      BenchCount = 0;
    }
    
    /* Compute the CPU load in active mode */
    for (i = 0; i <= BENCH_SIZE/2; i++)
    {
      Load = (Load + tBenchTable[i]) / 2;
      /* LCD_UsrLog(">Load = %d\n", tBenchTable[i]); */ /* Optional: display all values */
    }
    
    /* Display the benchmark results */
    LCD_UsrLog("> CPU Idle = %d\n", BenchRef);
    LCD_UsrLog("> CPU Active = %d\n", Load);
    LCD_UsrLog("> CPU Load = %f %%\n", (float)(((float)BenchRef-(float)Load)/(float)BenchRef) * 100);
    
    while (1)
    {
    }
#endif /* BENCHMARK_ENABLED */
  }
} 

/**
  * @brief  STM32_AudioRec_DataProcessSingle.
  *         This is an example of how to implement filtering or encoding functions
  *         into the audio recorder module. It simply increase volume by shifting data
  *         right. This function should process one single data at a time.
  * @param  pdata: pointer to the data to be prcessed
  * @param  fmt: number of bits in the data sample (ie. 16 or 8).
  * @param  chnbr: number of channels in the audio stream. It could be 1: Mono or 2: Stereo.
  * @retval 0 if all operations are OK. !0 else.
  */
uint32_t STM32_AudioRec_DataProcessSingle(uint8_t* pdata, uint32_t fmt, uint32_t chnbr)
{ 
  /* Check the size of data */
  if (fmt >= 16)
  {
    /* Increase volume */
    //*((uint16_t *)pdata) <<= 2;
    
    /* Get absolute value (without saturation) */
    //*((uint16_t *)pdata) &= 0x7FFF;
  }
  else
  {
    /* Increase volume */
    //*((uint8_t *)pdata) <<= 2;
    
    /* Get absolute value (without saturation) */
    //*((uint16_t *)pdata) &= 0x7F;
  }
  return 0;
}

/**
  * @brief  STM32_AudioRec_DataProcessBuffer.
  *         This is an example of how to implement filtering or encoding functions
  *         into the audio recorder module. It simply increase volume by shifting data
  *         right. This function should process one buffer at a time.
  * @param  pdata: pointer to the data buffer to be prcessed
  * @param  size: size of the data buffer to be prcessed (in bytes)
  * @param  fmt: number of bits in the data sample (ie. 16 or 8).
  * @param  chnbr: number of channels in the audio stream. It could be 1: Mono or 2: Stereo.
  * @retval 0 if all operations are OK. !0 else.
  */
uint32_t STM32_AudioRec_DataProcessBuffer(uint8_t * pbuf, uint32_t size, uint32_t fmt, uint32_t chnbr)
{
  
  return 0;
}

/**
  * @brief  Demo_SelectItem
  * @param  menu: pointer to the menu table
  * @param  item: index of the menu item to be selected
  * @param  size: total number of menu items
  * @retval None
  */
#ifndef NO_LCD  
static void Demo_SelectItem (uint8_t **menu , uint8_t item, uint8_t size)
{
  uint8_t i = 0;

  /* Highlight the selected item */
  LCD_SetTextColor(Cyan);
  LCD_SetBackColor(Blue2);
  LCD_DisplayStringLine(tLinesTab[item], menu [item]);

  /* Display the other items without highlight */
  LCD_SetTextColor(White);
  LCD_SetBackColor(Blue);
  for (i = 0; i < size; i++)
  {
    if (i != item)
    {
      LCD_DisplayStringLine(tLinesTab[i], menu [i]);
    }
  }

  /* Restore the background color configuration */
  LCD_SetBackColor(Black);
}
#endif /* NO_LCD */

#ifdef USE_FULL_ASSERT
/**
* @brief  assert_failed
*         Reports the name of the source file and the source line number
*         where the assert_param error has occurred.
* @param  File: pointer to the source file name
* @param  Line: assert_param error line source number
* @retval None
*/
void assert_failed(uint8_t* file, uint32_t line)
{
  /* User can add his own implementation to report the file name and line number,
  ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  
  /* Infinite loop */
  while (1)
  {}
}
#endif

/**
  * @}
  */ 


/**
  * @}
  */ 


/**
  * @}
  */ 

/******************* (C) COPYRIGHT 2012 STMicroelectronics *****END OF FILE****/
