/**
  ******************************************************************************
  * @file    stm32fxxx_it.c
  * @author  MCD Application Team
  * @version V1.0.0RC2
  * @date    13-February-2012
  * @brief   Main Interrupt Service Routines.
  *          This file provides all exceptions handler and peripherals interrupt
  *          service routine.
  ******************************************************************************
  * @attention
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * FOR MORE INFORMATION PLEASE READ CAREFULLY THE LICENSE AGREEMENT FILE
  * LOCATED IN THE ROOT DIRECTORY OF THIS FIRMWARE PACKAGE.
  *
  * <h2><center>&copy; COPYRIGHT 2012 STMicroelectronics</center></h2>
  ******************************************************************************
  */ 

/* Includes ------------------------------------------------------------------*/
#include "stm32fxxx_it.h"
#include "usb_core.h"
#include "usbd_core.h"

#if defined (USE_STM322xG_EVAL)
 #include "stm322xg_eval.h"
#elif defined(USE_STM324xG_EVAL)
 #include "stm324xg_eval.h"
#elif defined (USE_STM32F4XX_DISCOVERY)
 #include  "stm32f4_discovery.h"
#elif defined (USE_STM3210C_EVAL)
 #include "stm3210c_eval.h"
#else
 #error "Missing define: Evaluation board (ie. USE_STM322xG_EVAL)"
#endif /* USE_STM32F4XX_DISCOVERY */

#include "usbd_audio_core.h"

#if !defined(NO_LCD)
 #include "lcd_log.h"
#endif /* USE_STM32F4XX_DISCOVERY */

#include "stm32_audio.h"

#if defined (USE_STM322xG_EVAL)
 #include "stm322xg_audio_codec.h"
#elif defined(USE_STM324xG_EVAL)
 #include "stm324xg_audio_codec.h"
#elif defined (USE_STM3210C_EVAL)
 #include "stm3210c_audio_codec.h"
#elif defined (USE_STM32F4XX_DISCOVERY)
 #include "stm32F4_discovery_audio_codec.h"
#endif /* STM32F2XX */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
extern USB_OTG_CORE_HANDLE           USB_OTG_dev;
extern uint32_t USBD_OTG_ISR_Handler (USB_OTG_CORE_HANDLE *pdev);

#ifdef USB_OTG_HS_DEDICATED_EP1_ENABLED 
extern uint32_t USBD_OTG_EP1IN_ISR_Handler (USB_OTG_CORE_HANDLE *pdev);
extern uint32_t USBD_OTG_EP1OUT_ISR_Handler (USB_OTG_CORE_HANDLE *pdev);
#endif

extern AUDIO_FOPS_TypeDef  AUDIO_OUT_fops;

uint8_t* USER_INFORMATION1[6] = {
  ">STATE: Application Idle        ",
  ">STATE: Application Active      ",
  ">STATE: PLAYING                 ",
  ">STATE: PAUSED                  ",
  ">STATE: STOPPED                 ",
  ">STATE: Application Error !     ",
};

uint8_t* USER_INFORMATION2[2] = {
  ">Output: HEADPHONE                  ",
  ">Output: SPEAKER                    "
};

uint32_t PrevState = 0xFF;
uint32_t PrevOutput = OUTPUT_DEVICE_HEADPHONE;
uint32_t OutputState = OUTPUT_DEVICE_HEADPHONE;

#if defined(BENCHMARK_ENABLED)
 extern __IO uint32_t tBenchTable[BENCH_SIZE];
 extern __IO uint32_t BenchCount;
 extern __IO uint32_t BenchState;
#endif
/******************************************************************************/
/*            Cortex-M3 Processor Exceptions Handlers                         */
/******************************************************************************/

/**
  * @brief   This function handles NMI exception.
  * @param  None
  * @retval None
  */
void NMI_Handler(void)
{
}

/**
  * @brief  This function handles Hard Fault exception.
  * @param  None
  * @retval None
  */
void HardFault_Handler(void)
{
  /* Go to infinite loop when Hard Fault exception occurs */
  while (1)
  {
  }
}

/**
  * @brief  This function handles Memory Manage exception.
  * @param  None
  * @retval None
  */
void MemManage_Handler(void)
{
  /* Go to infinite loop when Memory Manage exception occurs */
  while (1)
  {
  }
}

/**
  * @brief  This function handles Bus Fault exception.
  * @param  None
  * @retval None
  */
void BusFault_Handler(void)
{
  /* Go to infinite loop when Bus Fault exception occurs */
  while (1)
  {
  }
}

/**
  * @brief  This function handles Usage Fault exception.
  * @param  None
  * @retval None
  */
void UsageFault_Handler(void)
{
  /* Go to infinite loop when Usage Fault exception occurs */
  while (1)
  {
  }
}

/**
  * @brief  This function handles SVCall exception.
  * @param  None
  * @retval None
  */
void SVC_Handler(void)
{
}

/**
  * @brief  This function handles Debug Monitor exception.
  * @param  None
  * @retval None
  */
void DebugMon_Handler(void)
{
}

/**
  * @brief  This function handles PendSVC exception.
  * @param  None
  * @retval None
  */
void PendSV_Handler(void)
{
}

/**
  * @brief  This function handles SysTick Handler.
  * @param  None
  * @retval None
  */
void SysTick_Handler(void)
{
#if defined(BENCHMARK_ENABLED)
  /* Reset the benchmark flag */
  BenchState = 0;

#else
 #if !defined(NO_LCD)
  /* Check if the audio player state has changed */
  if (PrevState != AUDIO_OUT_fops.GetState())
  {
    /* Information panel */
    LCD_SetTextColor(Green);
    LCD_DisplayStringLine( LCD_PIXEL_HEIGHT - 48, USER_INFORMATION1[AUDIO_OUT_fops.GetState()]);  
    LCD_SetTextColor(LCD_LOG_DEFAULT_COLOR);
    
    /* Update the previous state variable */
    PrevState = AUDIO_OUT_fops.GetState();
  }
  
  if (OutputState != PrevOutput)
  {
    Codec_SwitchOutput(OutputState);
    PrevOutput = OutputState;
    
    /* Information panel */
    LCD_SetTextColor(Green);
    if (OutputState == OUTPUT_DEVICE_HEADPHONE)
    {
      LCD_DisplayStringLine( LCD_PIXEL_HEIGHT - 32, USER_INFORMATION2[0]);
    }
    else
    {
      LCD_DisplayStringLine( LCD_PIXEL_HEIGHT - 32, USER_INFORMATION2[1]);
    }
    LCD_SetTextColor(LCD_LOG_DEFAULT_COLOR); 
  }
 #endif /* NO_LCD */
#endif /* BENCHMARK_ENABLED */
}

/**
  * @brief  This function handles EXTI15_10_IRQ Handler.
  * @param  None
  * @retval None
  */
#ifdef USE_USB_OTG_FS  
void OTG_FS_WKUP_IRQHandler(void)
{
  if(USB_OTG_dev.cfg.low_power)
  {
    *(uint32_t *)(0xE000ED10) &= 0xFFFFFFF9 ; 
    SystemInit();
    USB_OTG_UngateClock(&USB_OTG_dev);
  }
  EXTI_ClearITPendingBit(EXTI_Line18);
}
#endif

/**
  * @brief  This function handles EXTI15_10_IRQ Handler.
  * @param  None
  * @retval None
  */
#ifdef USE_USB_OTG_HS  
void OTG_HS_WKUP_IRQHandler(void)
{
  if(USB_OTG_dev.cfg.low_power)
  {
    *(uint32_t *)(0xE000ED10) &= 0xFFFFFFF9 ; 
    SystemInit();
    USB_OTG_UngateClock(&USB_OTG_dev);
  }
  EXTI_ClearITPendingBit(EXTI_Line20);
}
#endif

/**
  * @brief  This function handles OTG_HS Handler.
  * @param  None
  * @retval None
  */
#ifdef USE_USB_OTG_HS  
void OTG_HS_IRQHandler(void)
#else
void OTG_FS_IRQHandler(void)
#endif
{
  USBD_OTG_ISR_Handler (&USB_OTG_dev);
}

#ifdef USB_OTG_HS_DEDICATED_EP1_ENABLED 
/**
  * @brief  This function handles EP1_IN Handler.
  * @param  None
  * @retval None
  */
void OTG_HS_EP1_IN_IRQHandler(void)
{
  USBD_OTG_EP1IN_ISR_Handler (&USB_OTG_dev);
}

/**
  * @brief  This function handles EP1_OUT Handler.
  * @param  None
  * @retval None
  */
void OTG_HS_EP1_OUT_IRQHandler(void)
{
  USBD_OTG_EP1OUT_ISR_Handler (&USB_OTG_dev);
}
#endif

/**
  * @brief  This function handles EXTI15_10 interrupt request.
  * @param  None
  * @retval None
  */
#ifndef USE_STM32F4XX_DISCOVERY

#if defined (STM32F2XX) || defined (STM32F4XX)
void EXTI15_10_IRQHandler(void)
#elif defined(STM32F10X_CL)
void EXTI9_5_IRQHandler(void)
#endif /* STM32F2XX */
{
  static uint8_t FallingEdge = 0;
  
  /* Check if the interrupt source is the KEY EXTI line */
  if (EXTI_GetFlagStatus(KEY_BUTTON_EXTI_LINE) == SET)
  {
    /* Check if the push button is pushed or released (to ignore the button released event) */
    if (FallingEdge == 0)
    {
      /* Switch the output target between Speaker and Headphone */
      OutputState = (OutputState == OUTPUT_DEVICE_SPEAKER)? \
                    OUTPUT_DEVICE_HEADPHONE : OUTPUT_DEVICE_SPEAKER;
    
      /* Increment the variable to indicate that next interrupt corresponds to push
      button released event: Release event should be ignored */
      FallingEdge++;
    }
    else
    {
      /* Reset the variable to indicate that the next interrupt corresponds to push
      button pushed event. */
      FallingEdge = 0;
    }
    
    /* Clear the interrupt line flag */
    EXTI_ClearITPendingBit(KEY_BUTTON_EXTI_LINE);
  }
}
#endif /* USE_STM32F4XX_DISCOVERY */

/******************************************************************************/
/*                 STM32Fxxx Peripherals Interrupt Handlers                   */
/*  Add here the Interrupt Handler for the used peripheral(s) (PPP), for the  */
/*  available peripheral interrupt handler's name please refer to the startup */
/*  file (startup_stm32fxxx.s).                                               */
/******************************************************************************/

/**
  * @brief  This function handles PPP interrupt request.
  * @param  None
  * @retval None
  */
/*void PPP_IRQHandler(void)
{
}*/

/******************* (C) COPYRIGHT 2012 STMicroelectronics *****END OF FILE****/
