/**
  ******************************************************************************
  * @file    usbd_usr.c
  * @author  MCD Application Team
  * @version V1.0.0RC2
  * @date    13-February-2012
  * @brief   This file includes the user application layer
  ******************************************************************************
  * @attention
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * FOR MORE INFORMATION PLEASE READ CAREFULLY THE LICENSE AGREEMENT FILE
  * LOCATED IN THE ROOT DIRECTORY OF THIS FIRMWARE PACKAGE.
  *
  * <h2><center>&copy; COPYRIGHT 2012 STMicroelectronics</center></h2>
  ******************************************************************************
  */ 

/* Includes ------------------------------------------------------------------*/
#include "usbd_usr.h"
#include "usbd_ioreq.h"

#if !defined(NO_LCD)
 #include "lcd_log.h"
#endif /* USE_STM32F4XX_DISCOVERY */

#ifdef USE_STM322xG_EVAL
 #include "stm322xg_audio_codec.h"
#elif USE_STM324xG_EVAL
 #include "stm324xg_audio_codec.h"
#elif defined(USE_STM3210C_EVAL)
 #include "stm3210c_audio_codec.h"
#elif defined(USE_STM32F4XX_DISCOVERY)
 #include "stm32F4_discovery_audio_codec.h"
#endif /* USE_STM322xG_EVAL */

#include "usbd_audio_core.h"

/** @addtogroup STM32_USB_OTG_DEVICE_LIBRARY
* @{
*/

/** @defgroup USBD_USR 
* @brief    This file includes the user application layer
* @{
*/ 

/** @defgroup USBD_USR_Private_TypesDefinitions
* @{
*/ 
/**
* @}
*/ 


/** @defgroup USBD_USR_Private_Defines
* @{
*/ 
/**
* @}
*/ 


/** @defgroup USBD_USR_Private_Macros
* @{
*/ 
/**
* @}
*/ 


/** @defgroup USBD_USR_Private_Variables
* @{
*/ 

USBD_Usr_cb_TypeDef USR_cb =
{
  USBD_USR_Init,
  USBD_USR_DeviceReset,
  USBD_USR_DeviceConfigured,
  USBD_USR_DeviceSuspended,
  USBD_USR_DeviceResumed,
};

extern uint8_t* tMethods [];


/**
* @}
*/

/** @defgroup USBD_USR_Private_Constants
* @{
*/ 

/**
* @}
*/



/** @defgroup USBD_USR_Private_FunctionPrototypes
* @{
*/ 
/**
* @}
*/ 


/** @defgroup USBD_USR_Private_Functions
* @{
*/ 

#define USER_INFORMATION1      ">STATE: Application Idle            "
#define USER_INFORMATION2      "[Key]: Switch Headpone/Speaker      "

/**
* @brief  USBD_USR_Init 
*         Displays the message on LCD for host lib initialization
* @param  None
* @retval None
*/
void USBD_USR_Init(void)
{ 
 
  /* Initialize LEDs */
  STM_EVAL_LEDInit(LED1);
  STM_EVAL_LEDInit(LED2);
  STM_EVAL_LEDInit(LED3);
  STM_EVAL_LEDInit(LED4);   
  
#ifndef NO_LCD  
  /* Set the LCD header */
  LCD_LOG_SetHeader(tMethods[AUDIO_CORR_GetActiveMethod(DIRECTION_OUT)]);
  
#if defined(USB_AUDIO_V1_0)
  LCD_UsrLog("> Audio Class 1.0.\n");
#elif defined(USB_AUDIO_V2_0)
  LCD_UsrLog("> Audio Class 2.0.\n");
#endif /* USB_AUDIO_V1_0 */
  
  LCD_UsrLog("> USB device library started.\n"); 
  LCD_LOG_SetFooter ("    USB Audio Streaming v1.0.0rc1" ); 
  
  /* Information panel */
  LCD_SetTextColor(Green);
  LCD_DisplayStringLine( LCD_PIXEL_HEIGHT - 48, USER_INFORMATION1);
  LCD_SetTextColor(LCD_LOG_DEFAULT_COLOR); 

  /* Information panel */
  LCD_SetTextColor(Green);
  LCD_DisplayStringLine( LCD_PIXEL_HEIGHT - 32, USER_INFORMATION2);
  LCD_SetTextColor(LCD_LOG_DEFAULT_COLOR);   
#endif /* NO_LCD */
}

/**
* @brief  USBD_USR_DeviceReset 
*         Displays the message on LCD on device Reset Event
* @param  speed : device speed
* @retval None
*/
void USBD_USR_DeviceReset(uint8_t speed )
{
#ifndef NO_LCD  
 switch (speed)
 {
   case USB_OTG_SPEED_HIGH: 
     LCD_LOG_SetFooter ("     USB Audio Streaming v1.0.0rc1   [HS]" );
     break;

  case USB_OTG_SPEED_FULL: 
     LCD_LOG_SetFooter ("     USB Audio Streaming v1.0.0rc1   [FS]" );
     break;
 default:
     LCD_LOG_SetFooter ("     USB Audio Streaming v1.0.0rc1   [??]" );
     
 }
#endif /* NO_LCD */
}


/**
* @brief  USBD_USR_DeviceConfigured
*         Displays the message on LCD on device configuration Event
* @param  None
* @retval Staus
*/
void USBD_USR_DeviceConfigured (void)
{ 
#ifndef NO_LCD  
  LCD_UsrLog("> AUDIO Interface configured.\n");
#endif /* NO_LCD */
}
/**
* @brief  USBD_USR_DeviceSuspended 
*         Displays the message on LCD on device suspend Event
* @param  None
* @retval None
*/
void USBD_USR_DeviceSuspended(void)
{
#ifndef NO_LCD  
  LCD_UsrLog("> USB Device in Suspend Mode.\n");
#endif /* NO_LCD */

  /* Users can do their application actions here for the USB-Reset */
}


/**
* @brief  USBD_USR_DeviceResumed 
*         Displays the message on LCD on device resume Event
* @param  None
* @retval None
*/
void USBD_USR_DeviceResumed(void)
{
#ifndef NO_LCD  
    LCD_UsrLog("> USB Device in Idle Mode.\n");
#endif /* NO_LCD */

  /* Users can do their application actions here for the USB-Reset */
}



/**
  * @brief  EVAL_AUDIO_TransferComplete_CallBack
  * @param  None
  * @retval None
  */
void EVAL_AUDIO_TransferComplete_CallBack(uint32_t pBuffer, uint32_t Size)
{
}

/**
  * @brief  Codec_TIMEOUT_UserCallback
  * @param  None
  * @retval 0
  */
uint32_t Codec_TIMEOUT_UserCallback(void)
{
  I2C_InitTypeDef I2C_InitStructure;

#ifndef NO_LCD
  LCD_ErrLog("> I2C Timeout error (CS43L22)\n");
#endif /* NO_LCD */

  I2C_GenerateSTOP(I2C1, ENABLE);
  I2C_SoftwareResetCmd(I2C1, ENABLE);
  I2C_SoftwareResetCmd(I2C1, DISABLE);
    
  I2C_DeInit(I2C1);
  
  /* CODEC_I2C peripheral configuration */
  I2C_DeInit(I2C1);
  I2C_InitStructure.I2C_Mode = I2C_Mode_I2C;
  I2C_InitStructure.I2C_DutyCycle = I2C_DutyCycle_2;
  I2C_InitStructure.I2C_OwnAddress1 = 0x33;
  I2C_InitStructure.I2C_Ack = I2C_Ack_Enable;
  I2C_InitStructure.I2C_AcknowledgedAddress = I2C_AcknowledgedAddress_7bit;
  I2C_InitStructure.I2C_ClockSpeed = I2C_SPEED;
  /* Enable the I2C peripheral */
  I2C_Cmd(I2C1, ENABLE);  
  I2C_Init(I2C1, &I2C_InitStructure);  

#ifndef NO_LCD
  LCD_UsrLog("> I2C error recovered.\n");
#endif /* NO_LCD */
  
  return 0;
}

/**
  * @brief  CLK_SYNTH_TIMEOUT_UserCallback
  * @param  None
  * @retval 0
  */
uint32_t CLK_SYNTH_TIMEOUT_UserCallback(void)
{
  I2C_InitTypeDef I2C_InitStructure;

#ifndef NO_LCD
  LCD_ErrLog("> I2C Timeout error (CS2200)\n");
#endif /* NO_LCD */

  I2C_GenerateSTOP(I2C1, ENABLE);
  I2C_SoftwareResetCmd(I2C1, ENABLE);
  I2C_SoftwareResetCmd(I2C1, DISABLE);
    
  I2C_DeInit(I2C1);
  
  /* CODEC_I2C peripheral configuration */
  I2C_DeInit(I2C1);
  I2C_InitStructure.I2C_Mode = I2C_Mode_I2C;
  I2C_InitStructure.I2C_DutyCycle = I2C_DutyCycle_2;
  I2C_InitStructure.I2C_OwnAddress1 = 0x33;
  I2C_InitStructure.I2C_Ack = I2C_Ack_Enable;
  I2C_InitStructure.I2C_AcknowledgedAddress = I2C_AcknowledgedAddress_7bit;
  I2C_InitStructure.I2C_ClockSpeed = I2C_SPEED;
  /* Enable the I2C peripheral */
  I2C_Cmd(I2C1, ENABLE);  
  I2C_Init(I2C1, &I2C_InitStructure); 

#ifndef NO_LCD
  LCD_UsrLog("> I2C error recovered.\n");
#endif /* NO_LCD */
  
  return 0;
}

/**
* @}
*/ 

/**
* @}
*/ 

/******************* (C) COPYRIGHT 2012 STMicroelectronics *****END OF FILE****/
