/**
  *******************************************************************************
  * @file    stm32f4_discovery_audio_recorder.h
  * @author  MCD Application Team
  * @version V0.1.0
  * @date    05-January-2012
  * @brief    This file contains all the functions prototypes for the wave recorder
  *           firmware driver stm322xg_audio_recorder.c.
  ******************************************************************************          
  * @attention
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2011 STMicroelectronics</center></h2>
  ******************************************************************************  
  */

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __STM33F4_DISCOVERY_AUDIO_RECORDER_H
#define __STM33F4_DISCOVERY_AUDIO_RECORDER_H

#ifdef __cplusplus
extern "C"
{
#endif

/* Includes ------------------------------------------------------------------*/
#include "stm32f4xx.h"

#include "audio_app_conf.h"

/** @addtogroup Audio_Utilities
  * @{
  */

/** @addtogroup AUDIO_RECORDER
  * @{
  */

/** @defgroup AUDIO_RECORDER_Exported_Types
  * @{
  */
/**
  * @}
  */



/** @defgroup AUDIO_RECORDER_Exported_Constants
  * @{
  */

/* ----------------------------------------------------------------------------

   All the commented configuration parameters are gathered into audio_app_conf.h 
   file. If you already set them in audio_app_conf.h file, there is no need 
   to reconfigure them here.

-----------------------------------------------------------------------------*/
/* Set the preemption and subpriority values for the audio recorder timer interrupt */
/* #define AUDIO_REC_TIM_PRE_PRIO            1 */
/* #define AUDIO_REC_TIM_SUB_PRIO            0 */

/* #define AUDIO_REC_SPI_PRE_PRIO            3 */
/* #define AUDIO_REC_SPI_SUB_PRIO            1 */

/* Uncomment this define to enable single data processing (just after aquiring from ADC) */
//#define AUDIO_REC_USE_DATA_PROC_SINGLE
/* Uncomment this define to enable buffer data processing (just after a full buffer has 
   been filled in by ADC) */
//#define AUDIO_REC_USE_DATA_PROC_BUFFER

/* Maximum buffer size for the SPI data processing */
#define INTERNAL_BUFF_SIZE                30
  
/* TIMER Configuration defines */
#define AUDIO_REC_TIM                     TIM3
#define AUDIO_REC_TIM_IRQn                TIM3_IRQn
#define AUDIO_REC_TIM_IRQHANDLER          TIM3_IRQHandler

/* SPI Configuration defines */
#define SPI_SCK_PIN                       GPIO_Pin_10
#define SPI_SCK_GPIO_PORT                 GPIOB
#define SPI_SCK_GPIO_CLK                  RCC_AHB1Periph_GPIOB
#define SPI_SCK_SOURCE                    GPIO_PinSource10
#define SPI_SCK_AF                        GPIO_AF_SPI2

#define SPI_MOSI_PIN                      GPIO_Pin_3
#define SPI_MOSI_GPIO_PORT                GPIOC
#define SPI_MOSI_GPIO_CLK                 RCC_AHB1Periph_GPIOC
#define SPI_MOSI_SOURCE                   GPIO_PinSource3
#define SPI_MOSI_AF                       GPIO_AF_SPI2

#define AUDIO_REC_SPI_IRQHANDLER          SPI2_IRQHandler

/* Audio recorder clock update direction */
#define AUDIO_REC_DIRECTION_DOWN        0
#define AUDIO_REC_DIRECTION_UP          1


#define RELD_OFFSET                     4

#define SpeechReadAddr         0x0  /* Speech wave start read address */

#define _MAX_SS                255
#define REC_START_ADD          ((uint32_t)0x08004400)
#define REC_END_ADD            ((uint32_t)0x08100000)
#define REC_LENGTH             (uint32_t)(REC_END_ADD - REC_START_ADD)
#define REC_FREQ               8000 /* Audio revording frequency in Hz */


/**
  * @}
  */


/** @defgroup AUDIO_RECORDER_Exported_Macros
  * @{
  */
/**
  * @}
  */



/** @defgroup AUDIO_RECORDER_Exported_functions
  * @{
  */

uint32_t STM32_AudioRec_Init(uint32_t AudioFreq, uint32_t BitRes, uint32_t ChnlNbr);
uint32_t STM32_AudioRec_Start(uint8_t* pbuf, uint32_t size);
uint32_t STM32_AudioRec_Stop(void);
uint32_t STM32_AudioRec_HeaderInit(uint8_t* pHeadBuf);
uint32_t STM32_AudioRec_ClkUpdate(uint32_t Step, uint32_t Direction);

/* This function should be implemented by the user application.
   It is called into this driver when the current buffer is filled
   to prepare the next buffer pointer and its size. */
uint32_t STM32_AudioRec_Update(uint8_t** pbuf, uint32_t* pSize);

/* This function is called when new data samples are required.
   This function may be impelemented or modified by user depending 
   on the source of data to be sent to the recorder module. */
void STM32_AudioRec_GetSample(uint8_t* pBuf, uint32_t Nbr);

/* Optional data processing functions (Digital Filtering, Encoding ...).
   These functions' prototypes only are declared by the audio recorder module.
   Their implementation may be done (if needed) by the user into application modules. 
   These functions are enabled only when the relative defines:
   AUDIO_REC_USE_DATA_PROC_SINGLE and AUDIO_REC_USE_DATA_PROC_BUFFER are enabled in the
   file stm32f4_discovery_audio_conf.h  */
#ifdef AUDIO_REC_USE_DATA_PROC_SINGLE
 uint32_t STM32_AudioRec_DataProcessSingle(uint8_t* pdata, uint32_t fmt, uint32_t chnbr);
#endif

#ifdef AUDIO_REC_USE_DATA_PROC_BUFFER
 uint32_t STM32_AudioRec_DataProcessBuffer(uint8_t * pbuf, uint32_t size, uint32_t fmt, uint32_t chnbr);
#endif

#ifdef __cplusplus
}
#endif

#endif /*__STM33F4_DISCOVERY_AUDIO_RECORDER_H */

/**
  * @}
  */

/**
  * @}
  */

/**
  * @}
  */
  
/******************* (C) COPYRIGHT 2011 STMicroelectronics *****END OF FILE******/
