/**
  ******************************************************************************
  * @file    stm32f4_resampling.h
  * @author  MCD Application Team
  * @version V1.0.0
  * @date    27-January-2012
  * @brief   This file contains all the functions prototypes for the resampling
  *          driver.        
  ******************************************************************************
  * @attention
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * FOR MORE INFORMATION PLEASE READ CAREFULLY THE LICENSE AGREEMENT FILE
  * LOCATED IN THE ROOT DIRECTORY OF THIS FIRMWARE PACKAGE.
  *
  * <h2><center>&copy; COPYRIGHT 2012 STMicroelectronics</center></h2>
  ******************************************************************************
  */

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __STM32F4_RESAMPLING_H
#define __STM32F4_RESAMPLING_H

/* Includes ------------------------------------------------------------------*/
#include "stm32f4xx.h"

/* Exported types ------------------------------------------------------------*/
/* Exported constants --------------------------------------------------------*/
/* Max buffer size that can be passed to the SRC module (in bytes) */
#define MAX_BUFF_SZE        (192*4)

/* Resampling ratio, can be different for each  STM32_Resample_Process() call 
   Note: Size of  N (number of input samples) must be >= number of taps e.g. 32 */
#define RATIO_48_TO_48      268435456  //=   2^28
#define RATIO_48_TO_44      292838679  //=   48/44   * 2^28  = Fin/Fout*2^28
#define RATIO_44_TO_48      246065835  //=   44/48   * 2^28  = Fin/Fout*2^28
#define RATIO_48_TO_96      134217728  //=   48/96   * 2^28  = Fin/Fout*2^28
#define RATIO_48_TO_44_1    292174646  //=   48/44.1 * 2^28  = Fin/Fout*2^28
#define RATIO_48_TO_24      536870912  //=   48/24   * 2^28  = Fin/Fout*2^28
#define RATIO_48_TO_8       1610612736 //=   48/8    * 2^28  = Fin/Fout*2^28

#define RATIO_48_TO_48_1    267877378 //=   48/48.1 * 2^28  = Fin/Fout*2^28
#define RATIO_48_TO_47_8    269558617 //=   48/47.8 * 2^28  = Fin/Fout*2^28
#define RATIO_48_TO_49      262957181 //=   48/49   * 2^28  = Fin/Fout*2^28

/* SRC Interrupt routine (Do not Mdodify)(to modify the interrupt priorities, 
   call NVIC functions after STM32_Resample_Init() function call) */
#define SRC_IRQHandler      DMA2_Stream7_IRQHandler
#define SRC_IRQ             DMA2_Stream7_IRQn
#define SRC_IRQ_PREPRIO     0
#define SRC_IRQ_SUBPRIO     0

/* Exported macro ------------------------------------------------------------*/
/* Exported functions ------------------------------------------------------- */
uint32_t STM32_Resample_Init(uint32_t opt0, uint32_t opt1, uint32_t opt2, uint32_t opt3);
uint32_t STM32_Resample_Process(int* pSrcBuf, uint32_t Size, int* pDstBuf, uint32_t Ratio, uint32_t* pOutsize);

/* End of processing callback, called by SRC module and implemented by user application */
void STM32_Resample_ProcessCpltCallback(void);

#endif /* __STM32F4_RESAMPLING_H */

/******************* (C) COPYRIGHT 2012 STMicroelectronics *****END OF FILE****/
