/**
  ******************************************************************************
  * @file    stm32_audio.h
  * @author  MCD Application Team
  * @version V1.0.0RC1
  * @date    27-January-2012
  * @brief   Common header file for all audio processing.
  *
  *     *******************************************************************  
  *                           IMPORTANT NOTES
  *     *******************************************************************  
  *      This driver is intended for use ONLY in the scope of the current
  *      "STM32 USB Device Audio Streaming" Example.  
  *    
  ******************************************************************************
  * @attention
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * FOR MORE INFORMATION PLEASE READ CAREFULLY THE LICENSE AGREEMENT FILE
  * LOCATED IN THE ROOT DIRECTORY OF THIS FIRMWARE PACKAGE.
  *
  * <h2><center>&copy; COPYRIGHT 2012 STMicroelectronics</center></h2>
  ******************************************************************************
  */

#ifndef __STM32_AUDIO_H_
#define __STM32_AUDIO_H_

/* Includes ------------------------------------------------------------------*/
#ifdef STM32F2XX
 #include "stm32f2xx.h"
#elif defined(STM32F10X_CL)
 #include "stm32f10x.h"
#elif defined(STM32F4XX)
 #include "stm32f4xx.h"
#endif /* STM32F2XX */

/** @addtogroup STM32_Audio_Utilities
  * @{
  */
  
/** @defgroup STM32_AUDIO
  * @brief This file is the Header file for audio processor
  * @{
  */ 


/** @defgroup STM32_AUDIO_Exported_Defines
  * @{
  */ 

/* AudioFreq * DataSize (2 bytes) * NumChannels (Stereo: 2) */
#define DEFAULT_OUT_BIT_RESOLUTION                    16
#define DEFAULT_OUT_CHANNEL_NBR                       2 /* Mono = 1, Stereo = 2 */ 

/* AudioFreq * DataSize (2 bytes) * NumChannels (Stereo: 2) */
#define DEFAULT_IN_BIT_RESOLUTION                     16
#define DEFAULT_IN_CHANNEL_NBR                        1 /* Mono = 1, Stereo = 2 */

#define DIRECTION_OUT                                 0
#define DIRECTION_IN                                  1

/**
  * @}
  */ 


/** @defgroup STM32_AUDIO_Exported_TypesDefinitions
  * @{
  */
typedef struct _Audio_Fops
{
    uint8_t   (*Init)         (uint32_t  AudioFreq, uint32_t Volume, uint32_t options);
    uint8_t   (*DeInit)       (uint32_t options);
    uint8_t   (*AudioCmd)     (uint8_t* pbuf, uint32_t size, uint8_t cmd);
    uint8_t   (*VolumeCtl)    (uint8_t vol);
    uint8_t   (*MuteCtl)      (uint8_t cmd);
    uint32_t  (*OptionCtrl)   (uint8_t* cmd, uint32_t size);
    uint8_t   (*GetState)     (void);
    __IO uint32_t* pOptions;
}AUDIO_FOPS_TypeDef;

/**
  * @}
  */ 



/** @defgroup STM32_AUDIO_Exported_Macros
  * @{
  */ 
#define BYTE_0(val)                    (uint8_t)(val & 0xFF)
#define BYTE_1(val)                    (uint8_t)((val & 0xFF00) >> 8)
#define BYTE_2(val)                    (uint8_t)((val & 0xFF0000) >> 16)
#define BYTE_3(val)                    (uint8_t)((val & 0xFF000000) >> 24)

/**
  * @}
  */ 

/** @defgroup STM32_AUDIO_Exported_Variables
  * @{
  */ 

/**
  * @}
  */ 

/** @defgroup STM32_AUDIO_Exported_Functions
  * @{
  */

/* This function should be called by low layer functions when current buffer transfer is complete */
uint8_t  stm32_audio_BuffXferCplt (uint8_t Direction, uint8_t** pbuf, uint32_t* pSize);

/**
  * @}
  */ 

#endif  /* __STM32_AUDIO_H_ */
/**
  * @}
  */ 

/**
  * @}
  */ 
  
/******************* (C) COPYRIGHT 2012 STMicroelectronics *****END OF FILE****/
